package cz.filipnovak.wntr.service;

import cz.filipnovak.wntr.domain.ChatConversation;
import cz.filipnovak.wntr.domain.ChatMessage;
import cz.filipnovak.wntr.domain.User;
import cz.filipnovak.wntr.dto.ConversationDto;
import cz.filipnovak.wntr.dto.PostMessageDto;
import cz.filipnovak.wntr.enumeration.ValidationEnum;
import cz.filipnovak.wntr.repository.ChatConversationRepository;
import cz.filipnovak.wntr.repository.ChatMessageRepository;
import cz.filipnovak.wntr.utility.ValidationUtil;
import org.apache.commons.collections4.IterableUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class ChatService {

    @Autowired
    ChatConversationRepository conversationRepo;

    @Autowired
    ChatMessageRepository messageRepo;

    @Autowired
    UserService userService;

    @Autowired
    SecurityService securityService;

    public List<ChatConversation> listConversations() {
        return IterableUtils.toList(conversationRepo.findAll());
    }

    public ChatConversation startConversation(ConversationDto startConversation) {
        ValidationUtil.checkNull(startConversation.getId(), ValidationEnum.ILLEGAL_ARGUMENT);
        ValidationUtil.checkNotEmpty(startConversation.getSecret(), ValidationEnum.ILLEGAL_ARGUMENT);

        securityService.userStartsConversation();

        ChatConversation newConversation = new ChatConversation();
        newConversation.setSecret(startConversation.getSecret());

        return conversationRepo.save(newConversation);
    }

    public void postQuestion(PostMessageDto postMessage) {
        securityService.userPostsQuestion();
        postMessage(postMessage, null);
    }

    public void postAnswer(PostMessageDto postMessage) {
        securityService.userPostsAnswer();
        postMessage(postMessage, userService.findCurrentUser());
    }

    private void postMessage(PostMessageDto postMessage, User user) {
        ChatConversation conversation = ValidationUtil.getOrFail(
                conversationRepo.findById(postMessage.getConversationId()),
                ValidationEnum.ILLEGAL_ARGUMENT
        );

        ValidationUtil.checkEquals(
                conversation.getSecret(),
                postMessage.getConversationSecret(),
                ValidationEnum.ILLEGAL_ARGUMENT);

        ChatMessage chatMessage = new ChatMessage();
        chatMessage.setConversation(conversation);
        chatMessage.setText(postMessage.getMessageText());
        chatMessage.setUser(user);

        messageRepo.save(chatMessage);

        conversation.getMessages().add(chatMessage);
        conversation.setClosed(false);
        conversation.setAnswered(user != null);

        conversationRepo.save(conversation);
    }

    public List<ChatMessage> listMessages(ConversationDto conversationDto) {
        ChatConversation conversation = ValidationUtil.getOrFail(
                conversationRepo.findById(conversationDto.getId()),
                ValidationEnum.ILLEGAL_ARGUMENT
        );

        ValidationUtil.checkEquals(
                conversation.getSecret(),
                conversationDto.getSecret(),
                ValidationEnum.ILLEGAL_ARGUMENT);

        return conversation.getMessages();
    }

    public void closeConversation(Long conversationId) {
        ValidationUtil.checkNotNull(conversationId, ValidationEnum.ILLEGAL_ARGUMENT);

        securityService.userClosesConversation();

        ChatConversation conversation = ValidationUtil.getOrFail(
                conversationRepo.findById(conversationId),
                ValidationEnum.ILLEGAL_ARGUMENT
        );

        conversation.setClosed(true);
        conversationRepo.save(conversation);
    }
}
