package cz.filipnovak.wntr.service;

import cz.filipnovak.wntr.domain.Counter;
import cz.filipnovak.wntr.domain.CounterUpdate;
import cz.filipnovak.wntr.domain.Facility;
import cz.filipnovak.wntr.domain.User;
import cz.filipnovak.wntr.dto.OccupancyUpdateDto;
import cz.filipnovak.wntr.enumeration.ValidationEnum;
import cz.filipnovak.wntr.repository.CounterRepository;
import cz.filipnovak.wntr.repository.CounterUpdateRepository;
import cz.filipnovak.wntr.repository.FacilityRepository;
import cz.filipnovak.wntr.utility.SecurityUtil;
import cz.filipnovak.wntr.utility.StringUtil;
import cz.filipnovak.wntr.utility.ValidationUtil;
import org.apache.commons.collections4.IterableUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;
import java.util.NoSuchElementException;

@Service
public class FacilityService {

    private Logger logger = LoggerFactory.getLogger(FacilityService.class);

    @Autowired
    private FacilityRepository facilityRepository;

    @Autowired
    private CounterRepository counterRepository;

    @Autowired
    private CounterUpdateRepository counterUpdateRepository;

    @Autowired
    private UserService userService;

    @Autowired
    private SecurityService securityService;

    public List<Facility> findAll() {
        List<Facility> facilities = IterableUtils.toList(facilityRepository.findByOrderByName());
        return facilities;
    }

    public Facility findByPermanentId(String persistentId) {
        return facilityRepository
                .findByPersistentId(persistentId)
                .orElseThrow(NoSuchElementException::new);
    }

    /**
     * Creates facility.
     */
    @Transactional
    public void create(Facility facility) {
        logger.debug("User {} creates facility {}.", SecurityUtil.getUsername(), facility);

        ValidationUtil.checkNotNull(facility, ValidationEnum.ILLEGAL_ARGUMENT);
        ValidationUtil.checkNull(facility.getId(), ValidationEnum.CREATING_EXISTING);
        ValidationUtil.checkNull(facility.getPersistentId(), ValidationEnum.CREATING_EXISTING);

        securityService.userCreatesFacility();

        facility.setPersistentId(generateUniquePersistentId(facility.getName()));

        save(facility);
    }

    /**
     * Generates unique persistent id from given name.
     *
     * @param name not empty
     * @return not empty
     */
    public String generateUniquePersistentId(String name) {
        logger.debug("User {} generates persistent id.", SecurityUtil.getUsername());

        ValidationUtil.checkNotEmpty(name, ValidationEnum.ILLEGAL_ARGUMENT);

        return StringUtil.toPersistentId(name, facilityRepository::existsByPersistentId);
    }

    public void update(Facility facility) {
        logger.debug("User {} updates facility {}.", SecurityUtil.getUsername(), facility);

        ValidationUtil.checkNotNull(facility, ValidationEnum.ILLEGAL_ARGUMENT);
        ValidationUtil.checkNotNull(facility.getId(), ValidationEnum.UPDATING_NONEXISTENT);

        securityService.userUpdatesFacility(facility);

        save(facility);
    }

    /**
     * Saves facility.
     */
    private void save(Facility facility) {
        logger.trace("Saving facility {}.", facility);
        facilityRepository.save(facility);

        logger.debug("Saved facility {}.", facility);

        if (facility.getCounters() != null) {
            for (Counter counter : facility.getCounters()) {
                counter.setFacility(facility);
                save(counter, null);
            }
        }
    }

    /**
     * Saves counter along with it's state update.
     */
    private void save(Counter counter, String stamp) {
        logger.trace("Saving counter {} with stamp {}.", counter, stamp);
        counter.setLastUpdate(LocalDateTime.now());
        counterRepository.save(counter);

        CounterUpdate update = cloneState(counter, stamp);
        logger.trace("Saving update {}.", update);
        counterUpdateRepository.save(update);

        logger.debug("Counter {} saved with update {}.", counter, update);
    }

    private boolean stateAlreadySaved(Counter counter, String stamp) {
        // new counter can't be checked for existing states
        if (counter.getId() == null) {
            return false;
        }

        return counterUpdateRepository.findByCounterAndStamp(counter, stamp) != null;
    }

    private CounterUpdate cloneState(Counter counter, String stamp) {
        CounterUpdate facilityUpdate = new CounterUpdate();

        facilityUpdate.setCounter(counter);
        facilityUpdate.setOccupancy(counter.getOccupancy());
        facilityUpdate.setCapacity(counter.getCapacity());
        facilityUpdate.setTimeStamp(LocalDateTime.now());
        facilityUpdate.setStamp(stamp);

        return facilityUpdate;
    }

    /**
     * Returns facility of current user.
     */
    public Facility findCurrentUsersFacility() {
        logger.trace("Searching for facility of {}.", SecurityUtil.getUsername());

        Facility facility = userService
                .findCurrentUser()
                .getFacility();

        if (facility == null) {
            logger.trace("User {} has no facility.", SecurityUtil.getUsername());
            return null;
        }

        if (facility.isDeleted()) {
            logger.trace("Facility {} is deleted.", facility);
            return null;
        }

        logger.trace("Found facility {}.", facility);
        return facility;
    }

    /**
     * Updates occupancy with given increment.
     *
     * If increment was already used, then it does nothing.
     *
     * @param update
     */
    @Transactional(
            isolation = Isolation.SERIALIZABLE,
            propagation = Propagation.REQUIRED
    )
    public void updateOccupancyOfCurrentUsersFacility(OccupancyUpdateDto update) {
        String username = SecurityUtil.getUsername();
        logger.trace("Updating with {} by {}.", update, username);

        ValidationUtil.checkNotNull(update, ValidationEnum.ILLEGAL_ARGUMENT);

        User user = userService.findCurrentUser();

        Facility facility = user.getFacility();
        ValidationUtil.checkNotNull(facility, ValidationEnum.USERS_FACILITY_MISSING);

        securityService.userUpdatesOccupancy(facility);

        Counter counter = findByFacilityAndId(facility, update.getId());
        ValidationUtil.checkNotNull(counter, ValidationEnum.FACILITY_COUNTER_MISSING);

        String stamp = update.getStamp();
        if (stateAlreadySaved(counter, stamp)) {
            logger.warn("State of {} saved by {} with stamp={} was already saved.", counter, user, stamp);
            return;
        }

        Integer oldOccupancy = update.getValue() != null
                ? update.getValue()
                : counter.getOccupancy();
        Integer difference = update.getDifference() != null
                ? update.getDifference()
                : 0;
        Integer newOccupancy = oldOccupancy + difference;
        if (newOccupancy < 0) {
            newOccupancy = 0;
        }
        counter.setOccupancy(newOccupancy);

        save(counter, stamp);
    }

    public void delete(Facility facility) {
        logger.debug("User {} deletes facility {}.", SecurityUtil.getUsername(), facility);

        ValidationUtil.checkNotNull(facility, ValidationEnum.ILLEGAL_ARGUMENT);
        ValidationUtil.checkNotNull(facility.getId(), ValidationEnum.UPDATING_NONEXISTENT);

        securityService.userDeletesFacility(facility);

        facility.setDeleted(true);
        save(facility);
    }

    public Facility findById(Long id) {
        if (id == null) {
            return null;
        }

        return facilityRepository
                .findById(id)
                .orElse(null);
    }

    private Counter findByFacilityAndId(Facility facility, Long id) {
        if (id == null) {
            return null;
        }

        return facility.getCounters().stream()
                .filter(c -> id.equals(c.getId()))
                .findAny().orElse(null);
    }

    public void resetCounters() {
        for (Counter counter : counterRepository.findResettable()) {
            if (counter.getOccupancy() == 0) {
                continue; // nothing to reset
            }

            LocalDateTime resetAt = counter
                    .getResetAt()
                    .atDate(LocalDate.now());

            if (LocalDateTime.now().isBefore(resetAt)) {
                continue; // time has not come, yet
            }

            if (counter.getLastUpdate() != null && counter.getLastUpdate().isAfter(resetAt)) {
                continue; // already set
            }

            logger.info("Resetting counter {}.", counter);

            counter.setOccupancy(0);
            save(counter, "reset");
        }
    }
}
