package cz.filipnovak.wntr.service;

import cz.filipnovak.wntr.domain.FacilityType;
import cz.filipnovak.wntr.enumeration.ValidationEnum;
import cz.filipnovak.wntr.repository.FacilityTypeRepository;
import cz.filipnovak.wntr.utility.SecurityUtil;
import cz.filipnovak.wntr.utility.ValidationUtil;
import org.apache.commons.collections4.IterableUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.util.List;

@Service
public class FacilityTypeService {

    private Logger logger = LoggerFactory.getLogger(getClass());

    @Autowired
    FacilityTypeRepository facilityTypeRepository;

    @Autowired
    SecurityService securityService;

    public List<FacilityType> findAll() {
        return IterableUtils.toList(
                facilityTypeRepository.findAll()
        );
    }

    @Transactional
    public void create(FacilityType facilityType) {
        logger.debug("User {} creates facility type {}.", SecurityUtil.getUsername(), facilityType);

        ValidationUtil.checkNotNull(facilityType, ValidationEnum.ILLEGAL_ARGUMENT);
        ValidationUtil.checkNull(facilityType.getId(), ValidationEnum.CREATING_EXISTING);

        securityService.userCreatesFacilityType();

        saveAndReorder(facilityType);
    }

    private void saveAndReorder(FacilityType facilityType) {
        logger.trace("Saving facility type {}.", facilityType);
        facilityTypeRepository.save(facilityType);

        reorder(facilityType);

        logger.debug("Saved facility type {}.", facilityType);
    }

    /**
     * Reorders facility types.
     *
     * User specified position of given facility type. Others are shifted according to
     * the position. Position starts with 1, maximal number equals to the count of types.
     *
     * Deleted items are excluded.
     *
     * @param facilityType not null
     */
    private void reorder(FacilityType facilityType) {
        logger.trace("Reordering facilities.");

        List<FacilityType> types = facilityTypeRepository.findByDeletedFalseOrderByPositionAsc();

        if (!facilityType.isDeleted()) {
            types.removeIf(t -> t.getId().equals(facilityType.getId()));

            int newPosition = facilityType.getPosition().intValue() - 1;
            if (newPosition < 0) {
                newPosition = 0;
            }
            if (newPosition > types.size()) {
                newPosition = types.size();
            }
            types.add(newPosition, facilityType);
        }

        for (int i = 0; i < types.size(); i++) {
            types.get(i).setPosition(1L + i);
        }

        facilityTypeRepository.saveAll(types);
    }

    @Transactional
    public void update(FacilityType facilityType) {
        logger.debug("User {} updates facility type {}.", SecurityUtil.getUsername(), facilityType);

        ValidationUtil.checkNotNull(facilityType, ValidationEnum.ILLEGAL_ARGUMENT);
        ValidationUtil.checkNotNull(facilityType.getId(), ValidationEnum.UPDATING_NONEXISTENT);

        securityService.userUpdatesFacilityType(facilityType);

        saveAndReorder(facilityType);
    }

    public void delete(FacilityType facilityType) {
        logger.debug("User {} deletes facility type {}.", SecurityUtil.getUsername(), facilityType);

        ValidationUtil.checkNotNull(facilityType, ValidationEnum.ILLEGAL_ARGUMENT);
        ValidationUtil.checkNotNull(facilityType.getId(), ValidationEnum.UPDATING_NONEXISTENT);

        securityService.userDeletesFacilityType(facilityType);

        facilityType.setDeleted(true);
        saveAndReorder(facilityType);
    }
}
