package cz.filipnovak.wntr.service;

import cz.filipnovak.wntr.domain.Counter;
import cz.filipnovak.wntr.domain.Facility;
import cz.filipnovak.wntr.enumeration.ValidationEnum;
import cz.filipnovak.wntr.repository.CounterRepository;
import cz.filipnovak.wntr.repository.CounterUpdateRepository;
import cz.filipnovak.wntr.responsedto.LastUpdateDto;
import cz.filipnovak.wntr.utility.ValidationUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.stream.Collectors;

@Service
public class NotificationService {

    @Autowired
    CounterUpdateRepository counterUpdateRepository;

    @Autowired
    CounterRepository counterRepository;

    @Autowired
    SecurityService securityService;

    /**
     * Returns last updates of counters ordered by update time.
     *
     * Oldest updates are shown first. Motivation is to find which facilities (and their particular counters)
     * were not updated recently.
     */
    public List<LastUpdateDto> getCounterUpdates() {
        securityService.userListsCounterUpdates();

        return counterRepository
                .findByLastUpdate()
                .stream()
                .filter(counter -> counter.getFacility().isWinter())
                .map(LastUpdateDto::new)
                .collect(Collectors.toList());
    }

    /**
     * Snoozes last update by setting the value to current time.
     *
     * @param counterId not null
     */
    public void snoozeCounterUpdate(Long counterId) {
        ValidationUtil.checkNotNull(counterId, ValidationEnum.ILLEGAL_ARGUMENT);

        securityService.userSnoozesCounterUpdate();

        Counter counter = ValidationUtil.getOrFail(
                counterRepository.findById(counterId),
                ValidationEnum.ILLEGAL_ARGUMENT
        );

        counter.setLastUpdate(LocalDateTime.now());
        counterRepository.save(counter);
    }

    /**
     * Returns facilities with delayed update.
     */
    public List<LastUpdateDto> listExpiredUpdates(LocalDateTime time) {
        securityService.userListsCounterUpdates();

        return counterRepository
                .findByLastUpdate()
                .stream()
                .filter(counter -> counter.getFacility().isWinter())
                .filter(counter -> this.isExpired(counter, time))
                .map(LastUpdateDto::new)
                .collect(Collectors.toList());
    }

    private boolean isExpired(Counter counter, LocalDateTime time) {
        Facility facility = counter.getFacility();

        if (facility.getExpirationDecisiveTime() == null || facility.getExpirationNotificationTime() == null) {
            return false;
        }

        LocalDateTime decisiveTime = time
                .toLocalDate()
                .atTime(facility.getExpirationDecisiveTime());
        if (decisiveTime.isAfter(time)) {
            decisiveTime = decisiveTime.minusDays(1);
        }

        LocalDateTime notificationTime = decisiveTime
                .toLocalDate()
                .atTime(facility.getExpirationNotificationTime());
        if (notificationTime.isBefore(decisiveTime)) {
            notificationTime = notificationTime.plusDays(1);
        }

        boolean isExpired = counter.getLastUpdate().isBefore(decisiveTime);
        boolean shouldNotify = time.equals(notificationTime) || time.isAfter(notificationTime);

        return isExpired && shouldNotify;
    }
}
