package cz.filipnovak.wntr.service;

import cz.filipnovak.wntr.domain.Counter;
import cz.filipnovak.wntr.domain.CounterUpdate;
import cz.filipnovak.wntr.domain.Facility;
import cz.filipnovak.wntr.domain.OutreachRecord;
import cz.filipnovak.wntr.repository.CounterUpdateRepository;
import cz.filipnovak.wntr.repository.OccupancyByDate;
import cz.filipnovak.wntr.repository.OutreachRecordRepository;
import cz.filipnovak.wntr.utility.DateUtil;
import cz.filipnovak.wntr.utility.StringUtil;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFCellStyle;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.ServletOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.Map;
import java.util.stream.Collectors;

@Service
public class ReportService {

    @Autowired
    FacilityService facilityService;

    @Autowired
    CounterUpdateRepository counterUpdateRepository;

    @Autowired
    SecurityService securityService;

    @Autowired
    OutreachRecordRepository outreachRecordRepository;

    public void dailyMax(OutputStream outputStream) throws IOException {
        securityService.userViewsReport();

        LocalDate startDate = counterUpdateRepository.findOldestDate();
        LocalDate endDate = LocalDate.now().plusDays(1);

        XSSFWorkbook workbook = new XSSFWorkbook();

        XSSFCellStyle dateStyle = createDateStyle(workbook);
        XSSFCellStyle dateTimeStyle = createDateTimeStyle(workbook);

        XSSFSheet sheet = workbook.createSheet("Zimní opatření 2021-2022");

        int cellNum = 0;
        int rowNum = 0;

        XSSFRow row = sheet.createRow(rowNum++);
        XSSFCell cell = row.createCell(cellNum++);
        for (LocalDate date = startDate; date.isBefore(endDate); date = date.plusDays(1)) {
            cell = row.createCell(cellNum++);
            cell.setCellValue(date);
            cell.setCellStyle(dateStyle);

            sheet.autoSizeColumn(cellNum - 1);
        }

        for (Facility facility : facilityService.findAll()) {
            if (facility.isDeleted()) {
                continue;
            }

            for (Counter counter : facility.getCounters()) {
                if (counter.isDisabled() || StringUtil.isEmpty(counter.getTitle())) {
                    continue;
                }

                Map<LocalDate, Integer> max = counterUpdateRepository
                        .findDailyMaxOccupancyByCounter(counter)
                        .stream()
                        .collect(Collectors.toMap(
                                OccupancyByDate::getDate, OccupancyByDate::getOccupancy
                        ));

                Map<LocalDate, Integer> last = counterUpdateRepository
                        .findDailyLastOccupancyByCounter(counter.getId())
                        .stream()
                        .collect(Collectors.toMap(
                                t -> DateUtil.toLocalDate(t.get("date")).plusDays(1),
                                t -> (Integer) t.get("occupancy")
                        ));

                cellNum = 0;

                row = sheet.createRow(rowNum++);

                String title = facility.getName();
                if (!StringUtil.isEmpty(counter.getTitle())) {
                    title += " (" + counter.getTitle() + ")";
                }
                cell = row.createCell(cellNum++);
                cell.setCellValue(title);

                Integer lastOccupancy = counterUpdateRepository.findLastOccupancyBeforeDateByCounter(counter.getId(), startDate);
                for (LocalDate date = startDate; date.isBefore(endDate); date = date.plusDays(1)) {
                    lastOccupancy = last.getOrDefault(date, lastOccupancy);
                    Integer maxOccupancy = max.get(date);
                    Integer occupancy = Math.max(
                            lastOccupancy != null ? lastOccupancy : 0,
                            maxOccupancy != null ? maxOccupancy : 0);

                    cell = row.createCell(cellNum++);
                    cell.setCellValue(occupancy);
                }

                LocalDateTime newestTime = counterUpdateRepository.findNewestTime(counter);
                cell = row.createCell(cellNum++);
                cell.setCellValue(newestTime);
                cell.setCellStyle(dateTimeStyle);
            }
        }

        sheet.autoSizeColumn(0);
        sheet.autoSizeColumn(cellNum - 1);

        workbook.write(outputStream);
    }

    private XSSFCellStyle createDateTimeStyle(XSSFWorkbook workbook) {
        XSSFCellStyle style = workbook.createCellStyle();
        style.setDataFormat(
                workbook.getCreationHelper()
                        .createDataFormat()
                        .getFormat("d. m. yyyy h:mm:ss")
        );
        return style;
    }

    private XSSFCellStyle createDateStyle(XSSFWorkbook workbook) {
        XSSFCellStyle style = workbook.createCellStyle();
        style.setDataFormat(
                workbook.getCreationHelper()
                        .createDataFormat()
                        .getFormat("d. m. yyyy")
        );
        return style;
    }

    public void updates(ServletOutputStream outputStream) throws IOException {
        securityService.userViewsReport();

        XSSFWorkbook workbook = new XSSFWorkbook();
        XSSFCellStyle dateTimeStyle = createDateTimeStyle(workbook);

        XSSFSheet sheet = workbook.createSheet("Export všech změn");
        int rowNum = 0;

        XSSFRow headerRow = sheet.createRow(rowNum++);
        headerRow.createCell(0).setCellValue("Zařízení");
        headerRow.createCell(1).setCellValue("Počítadlo");
        headerRow.createCell(2).setCellValue("Datum a čas");
        headerRow.createCell(3).setCellValue("Kapacita");
        headerRow.createCell(4).setCellValue("Obsazenost");

        for (Facility facility : facilityService.findAll()) {
            if (facility.isDeleted()) {
                continue;
            }

            for (Counter counter : facility.getCounters()) {
                if (counter.isDisabled()) {
                    continue;
                }

                for (CounterUpdate counterUpdate : counter.getCounterUpdates()) {
                    XSSFRow row = sheet.createRow(rowNum++);

                    row.createCell(0).setCellValue(facility.getName());
                    row.createCell(1).setCellValue(counter.getTitle());

                    XSSFCell timeCell = row.createCell(2);
                    timeCell.setCellValue(counterUpdate.getTimeStamp());
                    timeCell.setCellStyle(dateTimeStyle);

                    if (counterUpdate.getCapacity() != null) {
                        row.createCell(3).setCellValue(counterUpdate.getCapacity());
                    }
                    if (counterUpdate.getOccupancy() != null) {
                        row.createCell(4).setCellValue(counterUpdate.getOccupancy());
                    }
                }
            }
        }

        for (int i = 0; i < 5; i++) {
            sheet.autoSizeColumn(i);
        }

        workbook.write(outputStream);
    }

    public void outreachRecords(ServletOutputStream outputStream) throws IOException {
        securityService.userViewsReport();

        XSSFWorkbook workbook = new XSSFWorkbook();
        XSSFCellStyle dateTimeStyle = createDateTimeStyle(workbook);

        XSSFSheet sheet = workbook.createSheet("Záznamy z terénu");
        int rowNum = 0;

        XSSFRow headerRow = sheet.createRow(rowNum++);
        headerRow.createCell(0).setCellValue("Datum");
        headerRow.createCell(1).setCellValue("Pracovník");
        headerRow.createCell(2).setCellValue("Organizace");
        headerRow.createCell(3).setCellValue("Místo kontaktu");
        headerRow.createCell(4).setCellValue("Potřeby klienta/charakteristika");
        headerRow.createCell(5).setCellValue("Pseudonym");
        headerRow.createCell(6).setCellValue("Záznam o řešení");

        for (OutreachRecord record : outreachRecordRepository.findByOrderByCreated()) {
            XSSFRow row = sheet.createRow(rowNum++);

            XSSFCell timeCell = row.createCell(0);
            timeCell.setCellValue(record.getCreated());
            timeCell.setCellStyle(dateTimeStyle);

            row.createCell(1).setCellValue(record.getUser().getName());
            row.createCell(2).setCellValue(record.getUser().getOrganization().getName());
            row.createCell(3).setCellValue(record.getLocation());
            row.createCell(4).setCellValue(record.getDescription());
            row.createCell(5).setCellValue(record.getNickname());
            row.createCell(6).setCellValue(record.getIntervention());
        }

        for (int i = 0; i < 7; i++) {
            sheet.autoSizeColumn(i);
        }

        workbook.write(outputStream);
    }
}
