package cz.filipnovak.wntr.service;

import cz.filipnovak.wntr.domain.Facility;
import cz.filipnovak.wntr.domain.FacilityType;
import cz.filipnovak.wntr.domain.User;
import cz.filipnovak.wntr.enumeration.PermissionEnum;
import cz.filipnovak.wntr.enumeration.ValidationEnum;
import cz.filipnovak.wntr.utility.SecurityUtil;
import cz.filipnovak.wntr.utility.ValidationUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class SecurityService {

    @Autowired
    UserService userService;

    public void userUpdatesFacility(Facility facility) {
        if (!canUserUpdateFacility(userService.findCurrentUser(), facility)) {
            throw new SecurityException("User " + getCurrentUsername() + " is not allowed to change facility " + facility + ".");
        }
    }

    private boolean canUserUpdateFacility(User user, Facility facility) {
        ValidationUtil.checkNotNull(facility, ValidationEnum.ILLEGAL_ARGUMENT);
        ValidationUtil.checkNotNull(facility.getId(), ValidationEnum.ILLEGAL_ARGUMENT);

        if (user.hasPermission(PermissionEnum.UPDATE_ANY_FACILITY)) {
            return true;
        }

        if (user.getFacility() == null) {
            return false;
        }

        if (!user.hasPermission(PermissionEnum.UPDATE_OWN_FACILITY)) {
            return false;
        }

        return facility.getId().equals(user.getFacility().getId());
    }

    public void userUpdatesOccupancy(Facility facility) {
        if (!canUserUpdateOccupancy(userService.findCurrentUser(), facility)) {
            throw new SecurityException("User " + getCurrentUsername() + " is not allowed to update occupancy on facility " + facility + ".");
        }
    }

    private boolean canUserUpdateOccupancy(User user, Facility facility) {
        if (canUserUpdateFacility(user, facility)) {
            return true;
        }

        if (!user.hasPermission(PermissionEnum.UPDATE_OCCUPANCY)) {
            return false;
        }

        return facility.getId().equals(user.getFacility().getId());
    }

    public void userCreatesFacility() {
        if (!canUserCreateFacility(userService.findCurrentUser())) {
            throw new SecurityException("User " + getCurrentUsername() + " is not allowed to create a facility.");
        }
    }

    private boolean canUserCreateFacility(User user) {
        return user.hasPermission(PermissionEnum.CREATE_FACILITY);
    }

    public void userDeletesFacility(Facility facility) {
        if (!canUserDeleteFacility(userService.findCurrentUser(), facility)) {
            throw new SecurityException("User " + getCurrentUsername() + " is not allowed to delete facility " + facility + ".");
        }
    }

    private boolean canUserDeleteFacility(User user, Facility facility) {
        return user.hasPermission(PermissionEnum.DELETE_FACILITY);
    }

    public void userCreatesFacilityType() {
        if (!canUserCreateFacilityType(userService.findCurrentUser())) {
            throw new SecurityException("User " + getCurrentUsername() + " is not allowd to create a facility type.");
        }
    }

    private boolean canUserCreateFacilityType(User user) {
        return user.hasPermission(PermissionEnum.CREATE_FACILITY_TYPE);
    }

    public void userUpdatesFacilityType(FacilityType facilityType) {
        if (!canUserUpdateFacilityType(userService.findCurrentUser())) {
            throw new SecurityException("User " + getCurrentUsername() + " is not allowd to update a facility type " + facilityType + ".");
        }
    }

    private boolean canUserUpdateFacilityType(User user) {
        return user.hasPermission(PermissionEnum.UPDATE_FACILITY_TYPE);
    }

    public void userDeletesFacilityType(FacilityType facilityType) {
        if (!canUserDeleteFacilityType(userService.findCurrentUser())) {
            throw new SecurityException("User " + getCurrentUsername() + " is not allowd to delete a facility type " + facilityType + ".");
        }
    }

    private boolean canUserDeleteFacilityType(User user) {
        return user.hasPermission(PermissionEnum.DELETE_FACILITY_TYPE);
    }

    public boolean canUserListUsers(User user) {
        return user.hasPermission(PermissionEnum.LIST_USERS);
    }

    public void userCreatesUser() {
        if (!canUserCreateUser(userService.findCurrentUser())) {
            throw new SecurityException("User " + getCurrentUsername() + " is not allowed to create a facility.");
        }
    }

    private boolean canUserCreateUser(User user) {
        return user.hasPermission(PermissionEnum.CREATE_USER);
    }

    public void userUpdatesUser(User updatedUser) {
        if (!canUserUpdateUser(userService.findCurrentUser(), updatedUser)) {
            throw new SecurityException("User " + getCurrentUsername() + " is not allowed to update user " + updatedUser + ".");
        }
    }

    private boolean canUserUpdateUser(User adminUser, User updatedUser) {
        ValidationUtil.checkNotNull(updatedUser, ValidationEnum.ILLEGAL_ARGUMENT);

        return adminUser.hasPermission(PermissionEnum.UPDATE_ANY_USER);
    }

    public void userDeletesUser(User deletedUser) {
        if (!canUserDeleteUser(userService.findCurrentUser(), deletedUser)) {
            throw new SecurityException("User " + getCurrentUsername() + " is not allowed to delete user " + deletedUser + ".");
        }
    }

    private boolean canUserDeleteUser(User adminUser, User deletedUser) {
        ValidationUtil.checkNotNull(deletedUser, ValidationEnum.ILLEGAL_ARGUMENT);

        // can't delete self
        if (adminUser.getId().equals(deletedUser.getId())) {
            return false;
        }

        return adminUser.hasPermission(PermissionEnum.DELETE_USER);
    }

    public void userChangesOwnPassword() {
        if (!canUserChangeOwnPassword(userService.findCurrentUser())) {
            throw new SecurityException("User " + getCurrentUsername() + " is not allowed to change his password.");
        }
    }

    private boolean canUserChangeOwnPassword(User user) {
        return user.hasPermission(PermissionEnum.CHANGE_OWN_PASSWORD);
    }

    public boolean canUserChangeAnyPassword(User user) {
        return user.hasPermission(PermissionEnum.CHANGE_ANY_PASSWORD);
    }

    public void userViewsReport() {
        if (!canUserViewReport(userService.findCurrentUser())) {
            throw new SecurityException("User " + getCurrentUsername() + " is not allowed to download a report.");
        }
    }

    private boolean canUserViewReport(User user) {
        return user.hasPermission(PermissionEnum.VIEW_REPORTS);
    }

    public void userStartsConversation() {
        if (!canUserStartConversation(userService.findCurrentUser())) {
            throw new SecurityException("User " + getCurrentUsername() + " can't start conversation.");
        }
    }

    private boolean canUserStartConversation(User user) {
        return user.isAnonymous();
    }

    public void userPostsQuestion() {
        if (!canUserPostQuestion(userService.findCurrentUser())) {
            throw new SecurityException("User " + getCurrentUsername() + " can't post question.");
        }
    }

    private boolean canUserPostQuestion(User user) {
        return user.isAnonymous();
    }

    public void userPostsAnswer() {
        if (!canUserPostAnswer(userService.findCurrentUser())) {
            throw new SecurityException("User " + getCurrentUsername() + " can't post answer.");
        }
    }

    private boolean canUserPostAnswer(User user) {
        return user.hasPermission(PermissionEnum.REPLY_TO_ANY_CHAT);
    }

    public void userClosesConversation() {
        if (!canUserCloseConversation(userService.findCurrentUser())) {
            throw new SecurityException("User " + getCurrentUsername() + " can't close conversation");
        }
    }

    private boolean canUserCloseConversation(User user) {
        return user.hasPermission(PermissionEnum.REPLY_TO_ANY_CHAT);
    }

    public void userSnoozesCounterUpdate() {
        if (!canUserSnoozeCounterUpdate(userService.findCurrentUser())) {
            throw new SecurityException("User " + getCurrentUsername() + " can't snooze counter update notification.");
        }
    }

    private boolean canUserSnoozeCounterUpdate(User user) {
        return user.hasPermission(PermissionEnum.SNOOZE_COUNTER_UPDATE_NOTIFICATION);
    }

    public void userListsCounterUpdates() {
        if (!canUserListCounterUpdates(userService.findCurrentUser())) {
            throw new SecurityException("User " + getCurrentUsername() + " can't list counter updates.");
        }
    }

    private boolean canUserListCounterUpdates(User user) {
        return user.hasPermission(PermissionEnum.VIEW_NOTIFICATIONS);
    }

    public void userListsAllOutreachRecords() {
        if (!canUserListAllOutreachRecords(userService.findCurrentUser())) {
            throw new SecurityException("User " + getCurrentUsername() + " can't list all outreach records.");
        }
    }

    private boolean canUserListAllOutreachRecords(User user) {
        return user.hasPermission(PermissionEnum.VIEW_ALL_OUTREACH_RECORDS);
    }

    public void userListsOwnOutreachRecords() {
        if (!canUserListOwnOutreachRecords(userService.findCurrentUser())) {
            throw new SecurityException("User " + getCurrentUsername() + " can't list own outreach records.");
        }
    }

    private boolean canUserListOwnOutreachRecords(User user) {
        return user.hasPermission(PermissionEnum.VIEW_OWN_OUTREACH_RECORDS);
    }

    private String getCurrentUsername() {
        return SecurityUtil.getUsername();
    }

    public void userCreatesOutreachRecord() {
        if (!canUserCreateOutreachRecord(userService.findCurrentUser())) {
            throw new SecurityException("User " + getCurrentUsername() + " can't create outreach record.");
        }
    }

    private boolean canUserCreateOutreachRecord(User user) {
        return user.hasPermission(PermissionEnum.CREATE_OUTREACH_RECORD);
    }
}
