package cz.filipnovak.wntr.service;

import cz.filipnovak.wntr.domain.Facility;
import cz.filipnovak.wntr.domain.Organization;
import cz.filipnovak.wntr.domain.User;
import cz.filipnovak.wntr.dto.ChangePasswordDto;
import cz.filipnovak.wntr.dto.UserDto;
import cz.filipnovak.wntr.enumeration.ValidationEnum;
import cz.filipnovak.wntr.repository.UserRepository;
import cz.filipnovak.wntr.utility.SecurityUtil;
import cz.filipnovak.wntr.utility.ValidationUtil;
import org.apache.commons.collections4.IterableUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Collections;
import java.util.List;

@Service
public class UserService {

    private Logger logger = LoggerFactory.getLogger(UserService.class);

    @Autowired
    private SecurityService securityService;

    @Autowired
    private UserRepository userRepository;

    @Autowired
    private OrganizationService organizationService;

    @Autowired
    private FacilityService facilityService;

    @Autowired
    private PasswordService passwordService;

    /**
     * Returns current user.
     */
    public User findCurrentUser() {
        String username = SecurityUtil.getUsername();
        ValidationUtil.checkNotEmpty(username, ValidationEnum.CURRENT_USERNAME_MISSING);

        if (SecurityUtil.ANONYMOUS_USER.equals(username)) {
            User user = new User();
            user.setUsername(SecurityUtil.ANONYMOUS_USER);
            user.setRoles(Collections.emptySet());
            return user;
        }

        User user = userRepository.findByUsername(username);
        ValidationUtil.checkNotNull(user, ValidationEnum.CURRENT_USER_MISSING);

        return user;
    }

    /**
     * Returns user with given username.
     */
    public User findUserByName(String username) {
        logger.trace("Finding user with username {}.", username);

        ValidationUtil.checkNotNull(username, ValidationEnum.USER_NOT_FOUND);

        User user = userRepository.findByUsername(username);
        logger.trace("Found user {}.", user);

        return user;
    }

    /**
     * User adminUsername creates user specified with createUser dto.
     */
    public void createUser(UserDto create) {
        logger.debug("User {} creates user {}.", SecurityUtil.getUsername(), create);

        ValidationUtil.checkNotNull(create, ValidationEnum.ILLEGAL_ARGUMENT);

        securityService.userCreatesUser();

        ValidationUtil.checkNull(userRepository.findByUsername(create.getUsername()), ValidationEnum.USER_EXISTS);

        User newUser = new User();

        newUser.setUsername(create.getUsername());

        String passwordHash = passwordService.hash(create.getPassword());
        newUser.setPasswordHash(passwordHash);

        newUser.setName(create.getName());

        Organization organization = organizationService.getOrCreate(create.getOrganization());
        newUser.setOrganization(organization);

        Facility facility = facilityService.findById(create.getFacilityId());
        newUser.setFacility(facility);

        newUser.setRoles(create.getRoles());

        userRepository.save(newUser);
    }

    /**
     * If permitted, it returns all users. Otherwise, only the current one.
     */
    public List<User> findAll() {
        logger.trace("User " + SecurityUtil.getUsername() + " lists all users.");

        if (securityService.canUserListUsers(findCurrentUser())) {
            return IterableUtils.toList(userRepository.findAll());
        } else {
            return Collections.singletonList(findCurrentUser());
        }
    }

    public User findUserByNameAndPassword(String username, String password) {
        ValidationUtil.checkNotEmpty(username, ValidationEnum.WRONG_CREDENTIALS);
        ValidationUtil.checkNotEmpty(password, ValidationEnum.WRONG_CREDENTIALS);

        User user = userRepository.findByUsername(username);
        ValidationUtil.checkNotNull(user, ValidationEnum.WRONG_CREDENTIALS);

        if (!passwordService.matches(password, user.getPasswordHash())) {
            ValidationUtil.fail(ValidationEnum.WRONG_CREDENTIALS);
        }

        return user;
    }

    public void changePassword(ChangePasswordDto changePassword) {
        logger.debug("User {} changes password of user {}.", SecurityUtil.getUsername(), changePassword);

        ValidationUtil.checkNotNull(changePassword, ValidationEnum.ILLEGAL_ARGUMENT);

        User updatedUser;
        if (securityService.canUserChangeAnyPassword(findCurrentUser())) {
            updatedUser = findUserByName(changePassword.getUsername());
        } else {
            securityService.userChangesOwnPassword();
            updatedUser = findUserByNameAndPassword(changePassword.getUsername(), changePassword.getCurrentPassword());
        }

        ValidationUtil.checkNotEmpty(changePassword.getNewPassword(), ValidationEnum.NEW_PASSWORD_WRONG);
        String passwordHash = passwordService.hash(changePassword.getNewPassword());
        updatedUser.setPasswordHash(passwordHash);

        userRepository.save(updatedUser);
    }

    public void updateUser(UserDto update) {
        logger.debug("User {} updates user {}.", SecurityUtil.getUsername(), update);

        ValidationUtil.checkNotNull(update, ValidationEnum.ILLEGAL_ARGUMENT);

        User user = userRepository.findByUsername(update.getUsername());
        ValidationUtil.checkNotNull(user, ValidationEnum.USER_NOT_FOUND);

        securityService.userUpdatesUser(user);

        user.setName(update.getName());

        Organization organization = organizationService.getOrCreate(update.getOrganization());
        user.setOrganization(organization);

        Facility facility = facilityService.findById(update.getFacilityId());
        user.setFacility(facility);

        user.setRoles(update.getRoles());

        userRepository.save(user);
    }

    public void delete(UserDto delete) {
        logger.debug("User {} deletes user {}.", SecurityUtil.getUsername(), delete);

        ValidationUtil.checkNotNull(delete, ValidationEnum.ILLEGAL_ARGUMENT);

        User user = userRepository.findByUsername(delete.getUsername());
        ValidationUtil.checkNotNull(user, ValidationEnum.USER_NOT_FOUND);

        securityService.userDeletesUser(user);

        user.setDeleted(true);

        userRepository.save(user);
    }
}
