package cz.filipnovak.wntr.service;

import cz.filipnovak.wntr.domain.ChatConversation;
import cz.filipnovak.wntr.domain.ChatMessage;
import cz.filipnovak.wntr.domain.User;
import cz.filipnovak.wntr.dto.ConversationDto;
import cz.filipnovak.wntr.dto.PostMessageDto;
import cz.filipnovak.wntr.enumeration.RoleEnum;
import cz.filipnovak.wntr.exception.ValidationException;
import cz.filipnovak.wntr.repository.ChatConversationRepository;
import cz.filipnovak.wntr.repository.UserRepository;
import cz.filipnovak.wntr.utility.SecurityUtil;
import org.junit.jupiter.api.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.annotation.Rollback;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collections;
import java.util.List;

import static org.junit.jupiter.api.Assertions.*;

@RunWith(SpringRunner.class)
@SpringBootTest
@ActiveProfiles("test")
@Transactional
@Rollback
@EnableAutoConfiguration
class ChatServiceTest {

    @Autowired
    private ChatService chatService;

    @Autowired
    ChatConversationRepository conversationRepo;

    @Autowired
    UserRepository userRepository;

    @Test
    void listConversations() {
        List<ChatConversation> conversations = chatService.listConversations();
        assertNotNull(conversations);
        assertTrue(conversations.isEmpty());

        conversationRepo.save(new ChatConversation());
        conversationRepo.save(new ChatConversation());

        assertEquals(2, chatService.listConversations().size());
    }

    @Test
    void startConversation() {
        assertThrows(ValidationException.class, () -> {
            ConversationDto c = new ConversationDto(null, null);
            chatService.startConversation(c);
        });

        assertThrows(ValidationException.class, () -> {
            ConversationDto c = new ConversationDto(1l, null);
            chatService.startConversation(c);
        });

        assertThrows(ValidationException.class, () -> {
            ConversationDto c = new ConversationDto(1l, "test");
            chatService.startConversation(c);
        });

        assertEquals(0, chatService.listConversations().size());

        ConversationDto c = new ConversationDto(null, "test");
        chatService.startConversation(c);

        List<ChatConversation> conversations = chatService.listConversations();
        assertEquals(1, conversations.size());
        assertEquals("test", conversations.get(0).getSecret());
        assertTrue(conversations.get(0).getMessages().isEmpty());
    }

    @Test
    void postMessage() {
        assertThrows(ValidationException.class, () -> {
            PostMessageDto messageDto = new PostMessageDto(
                    null, null, "Test text");
            chatService.postQuestion(messageDto);
        });

        assertThrows(ValidationException.class, () -> {
            PostMessageDto messageDto = new PostMessageDto(
                    1L, null, "Test text");
            chatService.postQuestion(messageDto);
        });

        assertThrows(ValidationException.class, () -> {
            PostMessageDto messageDto = new PostMessageDto(
                    null, "test", "Test text");
            chatService.postQuestion(messageDto);
        });

        assertThrows(ValidationException.class, () -> {
            PostMessageDto messageDto = new PostMessageDto(
                    1L, "test", "Test text");
            chatService.postQuestion(messageDto);
        });

        ChatConversation conversation = chatService.startConversation(new ConversationDto(null, "test"));

        assertThrows(ValidationException.class, () -> {
            PostMessageDto messageDto = new PostMessageDto(
                    null, null, "Test text");
            chatService.postQuestion(messageDto);
        });

        assertThrows(ValidationException.class, () -> {
            PostMessageDto messageDto = new PostMessageDto(
                    conversation.getId(), null, "Test text");
            chatService.postQuestion(messageDto);
        });

        assertThrows(ValidationException.class, () -> {
            PostMessageDto messageDto = new PostMessageDto(
                    null, conversation.getSecret(), "Test text");
            chatService.postQuestion(messageDto);
        });

        PostMessageDto messageDto = new PostMessageDto(
                conversation.getId(), conversation.getSecret(), "Test text");
        chatService.postQuestion(messageDto);

        List<ChatMessage> messages = chatService.listMessages(
                new ConversationDto(conversation.getId(), conversation.getSecret()));
        assertEquals(1, messages.size());
        assertEquals("Test text", messages.get(0).getText());
        assertNull(messages.get(0).getUser());

        User user = new User();
        user.setUsername("test_user");
        user.setRoles(Collections.singleton(RoleEnum.FACILITY_ADMIN));
        userRepository.save(user);
        SecurityUtil.setUsername("test_user");

        messageDto = new PostMessageDto(
                conversation.getId(), conversation.getSecret(), "Test reply");
        chatService.postAnswer(messageDto);

        messages = chatService.listMessages(
                new ConversationDto(conversation.getId(), conversation.getSecret())
        );
        assertEquals(2, messages.size());
        assertEquals("Test reply", messages.get(1).getText());
        assertEquals("test_user", messages.get(1).getUser().getUsername());
    }

    @Test
    void listMessages() {
        ChatConversation c = chatService.startConversation(new ConversationDto(null, "xxx"));
        chatService.postQuestion(new PostMessageDto(c.getId(), c.getSecret(), "Hello, krtko!"));
        chatService.postQuestion(new PostMessageDto(c.getId(), c.getSecret(), "Hello, krtko!"));

        List<ChatMessage> chatMessages = chatService.listMessages(new ConversationDto(c.getId(), c.getSecret()));
        assertNotNull(chatMessages.get(0).getPosted());
    }
}