package cz.filipnovak.wntr.service;

import cz.filipnovak.wntr.domain.Counter;
import cz.filipnovak.wntr.domain.Facility;
import cz.filipnovak.wntr.domain.User;
import cz.filipnovak.wntr.dto.OccupancyUpdateDto;
import cz.filipnovak.wntr.enumeration.RoleEnum;
import cz.filipnovak.wntr.enumeration.ValidationEnum;
import cz.filipnovak.wntr.exception.ValidationException;
import cz.filipnovak.wntr.repository.CounterRepository;
import cz.filipnovak.wntr.repository.FacilityRepository;
import cz.filipnovak.wntr.repository.UserRepository;
import cz.filipnovak.wntr.utility.SecurityUtil;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.annotation.Rollback;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalTime;
import java.util.*;

import static org.junit.jupiter.api.Assertions.*;

@RunWith(SpringRunner.class)
@SpringBootTest
@ActiveProfiles("test")
@Transactional
@Rollback
@EnableAutoConfiguration
class FacilityServiceTest {

    private static final String ADMIN = "admin";
    private static final String USER_WITH_FACILITY = "tester_facility";
    private static final String USER_WITHOUT_FACILITY = "tester_without_facility";
    private static final String NONEXISTENT_USER = "tester_zero";

    private static Long counterId;

    @Autowired
    private FacilityService facilityService;

    @Autowired
    private FacilityRepository facilityRepository;

    @Autowired
    UserRepository userRepository;

    @Autowired
    private CounterRepository counterRepository;

    @BeforeEach
    public void createTestUsers() {
        User admin = new User();
        admin.setUsername(ADMIN);
        admin.setRoles(new HashSet<>(Arrays.asList(
                RoleEnum.USER_ADMIN,
                RoleEnum.FACILITY_ADMIN
        )));
        userRepository.save(admin);

        SecurityUtil.setUsername(ADMIN);

        Counter counter = new Counter();
        counter.setTitle("muži");

        List<Counter> counters = new java.util.ArrayList<>();
        counters.add(counter);

        Facility facility = new Facility();
        facility.setName("testing facility");
        facility.setCounters(counters);
        facilityService.create(facility);

        counterId = facility.getCounters().get(0).getId();

        User userWithFacility = new User();
        userWithFacility.setUsername(USER_WITH_FACILITY);
        userWithFacility.setFacility(facility);
        userWithFacility.setRoles(Collections.singleton(RoleEnum.FACILITY_MANAGER));
        userRepository.save(userWithFacility);

        User userWithoutFacility = new User();
        userWithoutFacility.setUsername(USER_WITHOUT_FACILITY);
        userWithoutFacility.setRoles(Collections.emptySet());
        userRepository.save(userWithoutFacility);
    }

    @Test
    void findAll() {
        int originalSize = facilityService.findAll().size();

        Facility facility = new Facility();
        facility.setName("Testing facility");
        facility.setEmail("email1@seznam.cz");
        facilityService.create(facility);

        facility = new Facility();
        facility.setName("Testing facility");
        facility.setEmail("email2@seznam.cz");
        facilityService.create(facility);

        List<Facility> all = facilityService.findAll();
        assertNotNull(all);

        assertEquals(originalSize + 2, all.size());
    }

    @Test
    void create() {
        String email = "bla@bla.cz";

        Facility facility = new Facility();
        facility.setName("Testing facility");
        facility.setEmail(email);

        facilityService.create(facility);
        Long id = facility.getId();

        Optional<Facility> byId = facilityRepository.findById(id);
        assertTrue(byId.isPresent());

        assertEquals(email, byId.get().getEmail());

        try {
            facilityService.create(facility);
            fail("Can't create existing facility.");
        } catch (ValidationException e) {
            assertSame(e.getType(), ValidationEnum.CREATING_EXISTING);
        }

        facility = new Facility();

        SecurityUtil.setUsername(null);
        try {
            facilityService.create(facility);
            fail("Can't create facility without security context.");
        } catch (Exception e) {
            assertTrue(e instanceof SecurityException);
        }

        SecurityUtil.setUsername(USER_WITH_FACILITY);
        try {
            facilityService.create(facility);
            fail("Non admin should not create a facility.");
        } catch (Exception e) {
            assertTrue(e instanceof SecurityException);
        }

        SecurityUtil.setUsername(NONEXISTENT_USER);
        try {
            facilityService.create(facility);
            fail("Non existent user should not create a facility.");
        } catch (ValidationException e) {
            assertSame(e.getType(), ValidationEnum.CURRENT_USER_MISSING);
        }
    }

    @Test
    void generateUniquePersistentId() {
        String id = facilityService.generateUniquePersistentId("test");
        assertEquals("test", id);

        Facility facility = new Facility();
        facility.setName("Test");
        facilityService.create(facility);

        id = facilityService.generateUniquePersistentId("test");
        assertEquals("test-2", id);

        facility = new Facility();
        facility.setName("Test");
        facilityService.create(facility);

        id = facilityService.generateUniquePersistentId("test");
        assertEquals("test-3", id);

        id = facilityService.generateUniquePersistentId("test_2");
        assertEquals("test-2-2", id);
    }

    @Test
    void updateOccupancy() {
        // increment (from null/0 to 1)

        OccupancyUpdateDto increment = new OccupancyUpdateDto();
        increment.setId(counterId);
        increment.setDifference(1);
        increment.setStamp("blabla");

        SecurityUtil.setUsername(USER_WITH_FACILITY);
        facilityService.updateOccupancyOfCurrentUsersFacility(increment);

        Facility facility = facilityService.findCurrentUsersFacility();
        assertEquals(1, facility.getCounters().get(0).getOccupancy());

        // increment with duplicate stamp (no change)

        increment = new OccupancyUpdateDto();
        increment.setId(counterId);
        increment.setDifference(1);
        increment.setStamp("blabla");

        facilityService.updateOccupancyOfCurrentUsersFacility(increment);

        facility = facilityService.findCurrentUsersFacility();
        assertEquals(1, facility.getCounters().get(0).getOccupancy());

        // increment with unique stamp (from 1 to 2)

        increment.setStamp("neblabla");
        facilityService.updateOccupancyOfCurrentUsersFacility(increment);

        facility = facilityService.findCurrentUsersFacility();
        assertEquals(2, facility.getCounters().get(0).getOccupancy());

        // decrement (from 2 to 0)

        OccupancyUpdateDto decrement = new OccupancyUpdateDto();
        decrement.setId(counterId);
        decrement.setDifference(-2);
        decrement.setStamp("d1");
        facilityService.updateOccupancyOfCurrentUsersFacility(decrement);

        facility = facilityService.findCurrentUsersFacility();
        assertEquals(0, facility.getCounters().get(0).getOccupancy());

        // decrement bellow zero (from 0 to -1)

        decrement.setDifference(-1);
        decrement.setStamp("d2");
        facilityService.updateOccupancyOfCurrentUsersFacility(decrement);

        facility = facilityService.findCurrentUsersFacility();
        assertEquals(0, facility.getCounters().get(0).getOccupancy());

        // decrement too much (from 10 to -2)

        facility.getCounters().get(0).setOccupancy(10);
        facilityService.update(facility);

        facility = facilityService.findCurrentUsersFacility();
        assertEquals(10, facility.getCounters().get(0).getOccupancy());

        decrement.setDifference(-12);
        decrement.setStamp("d3");
        facilityService.updateOccupancyOfCurrentUsersFacility(decrement);

        facility = facilityService.findCurrentUsersFacility();
        assertEquals(0, facility.getCounters().get(0).getOccupancy());
    }

    @Test
    void findUsersFacility() {
        SecurityUtil.setUsername(USER_WITH_FACILITY);
        Facility facility = facilityService.findCurrentUsersFacility();
        assertNotNull(facility);

        SecurityUtil.setUsername(USER_WITHOUT_FACILITY);
        facility = facilityService.findCurrentUsersFacility();
        assertNull(facility);
    }

    @Test
    void resetCounters() {
        SecurityUtil.setUsername(USER_WITH_FACILITY);

        // counter with occupancy == 0 and resetAt == null should not change

        assertEquals(0, facilityService
                .findCurrentUsersFacility()
                .getCounters().get(0)
                .getOccupancy());

        facilityService.resetCounters();

        assertEquals(0, facilityService
                .findCurrentUsersFacility()
                .getCounters().get(0)
                .getOccupancy());

        // counter with resetAt == null should not change

        OccupancyUpdateDto update = new OccupancyUpdateDto();
        update.setId(
                facilityService
                        .findCurrentUsersFacility()
                        .getCounters().get(0)
                        .getId()
        );
        update.setDifference(1);
        update.setStamp("1");
        facilityService.updateOccupancyOfCurrentUsersFacility(update);

        assertEquals(1, facilityService
                .findCurrentUsersFacility()
                .getCounters().get(0)
                .getOccupancy()
        );

        facilityService.resetCounters();

        assertEquals(1, facilityService
                .findCurrentUsersFacility()
                .getCounters().get(0)
                .getOccupancy()
        );

        // counter with resetAt in future should not change

        LocalTime resetInFuture = LocalTime.now().plusSeconds(1);
        Counter counter = facilityService
                .findCurrentUsersFacility()
                .getCounters().get(0);
        counter.setResetAt(resetInFuture);
        counterRepository.save(counter);

        facilityService.resetCounters();

        assertEquals(1, facilityService
                .findCurrentUsersFacility()
                .getCounters().get(0)
                .getOccupancy()
        );

        // counter with resetAt in past should reset

        LocalTime resetInPast = LocalTime.now();
        counter = facilityService
                .findCurrentUsersFacility()
                .getCounters().get(0);
        counter.setResetAt(resetInPast);
        counterRepository.save(counter);

        facilityService.resetCounters();

        assertEquals(0, facilityService
                .findCurrentUsersFacility()
                .getCounters().get(0)
                .getOccupancy()
        );

        // counter with update after resetAt should not reset

        update = new OccupancyUpdateDto();
        update.setId(
                facilityService
                        .findCurrentUsersFacility()
                        .getCounters().get(0)
                        .getId()
        );
        update.setDifference(1);
        update.setStamp("2");
        facilityService.updateOccupancyOfCurrentUsersFacility(update);

        resetInPast = LocalTime.now();
        counter = facilityService
                .findCurrentUsersFacility()
                .getCounters().get(0);
        counter.setResetAt(resetInPast);
        counterRepository.save(counter);

        update = new OccupancyUpdateDto();
        update.setId(
                facilityService
                        .findCurrentUsersFacility()
                        .getCounters().get(0)
                        .getId()
        );
        update.setDifference(1);
        update.setStamp("3");
        facilityService.updateOccupancyOfCurrentUsersFacility(update);

        facilityService.resetCounters();

        assertEquals(2, facilityService
                .findCurrentUsersFacility()
                .getCounters().get(0)
                .getOccupancy()
        );
    }
}