package cz.filipnovak.wntr.service;

import cz.filipnovak.wntr.domain.OutreachRecord;
import cz.filipnovak.wntr.domain.User;
import cz.filipnovak.wntr.enumeration.RoleEnum;
import cz.filipnovak.wntr.repository.OutreachRecordRepository;
import cz.filipnovak.wntr.repository.UserRepository;
import cz.filipnovak.wntr.utility.SecurityUtil;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.annotation.Rollback;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.transaction.annotation.Transactional;

import java.time.Duration;
import java.time.LocalDateTime;
import java.util.Collections;
import java.util.List;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;

@RunWith(SpringRunner.class)
@SpringBootTest
@ActiveProfiles("test")
@Transactional
@Rollback
@EnableAutoConfiguration
class OutreachServiceTest {

    private static final String TEST_USER = "test_user";

    private static final String TEST_ADMIN = "test_admin";

    @Autowired
    OutreachRecordRepository outreachRecordRepo;

    @Autowired
    OutreachService outreachService;

    @Autowired
    UserRepository userRepository;

    @BeforeEach
    void prepare() {
        if (userRepository.findByUsername(TEST_USER) == null) {
            User user = new User();
            user.setUsername(TEST_USER);
            user.setRoles(Collections.singleton(RoleEnum.OUTREACH_WORKER));
            userRepository.save(user);
        }

        if (userRepository.findByUsername(TEST_ADMIN) == null) {
            User user = new User();
            user.setUsername(TEST_ADMIN);
            user.setRoles(Collections.singleton(RoleEnum.FACILITY_ADMIN));
            userRepository.save(user);
        }

        outreachRecordRepo.deleteAll();

        for (
                LocalDateTime created = LocalDateTime.now();
                created.isAfter(LocalDateTime.now().minusWeeks(3));
                created = created.minusHours(1)
        ) {
            OutreachRecord record = new OutreachRecord();
            record.setUser(userRepository.findByUsername(TEST_USER));
            record.setCreated(created);
            outreachRecordRepo.save(record);
        }
    }

    @Test
    void listLatestRecords() {
        assertThrows(SecurityException.class, this::listLastDay);

        SecurityUtil.setUsername(TEST_USER);
        assertThrows(SecurityException.class, this::listLastDay);

        SecurityUtil.setUsername(TEST_ADMIN);

        List<OutreachRecord> records = listLastDay();
        assertEquals(24, records.size());
    }

    private List<OutreachRecord> listLastDay() {
        return outreachService.listLatestRecords(Duration.ofHours(24));
    }

    @Test
    void listLatestRecordsOfCurrentUser() {
        assertThrows(SecurityException.class, this::listLastTwoWeeks);

        SecurityUtil.setUsername(TEST_ADMIN);
        assertThrows(SecurityException.class, this::listLastTwoWeeks);

        SecurityUtil.setUsername(TEST_USER);

        List<OutreachRecord> records = listLastTwoWeeks();
        assertEquals(24 * 7 * 2, records.size());
    }

    private List<OutreachRecord> listLastTwoWeeks() {
        return outreachService.listLatestRecordsOfCurrentUser(Duration.ofDays(14));
    }

    @Test
    void createRecord() {
        assertThrows(
                SecurityException.class,
                () -> outreachService.createRecord(new OutreachRecord())
        );

        SecurityUtil.setUsername(TEST_ADMIN);

        assertThrows(
                SecurityException.class,
                () -> outreachService.createRecord(new OutreachRecord())
        );

        SecurityUtil.setUsername(TEST_USER);

        outreachService.createRecord(new OutreachRecord());
        outreachService.createRecord(new OutreachRecord());
        outreachService.createRecord(new OutreachRecord());

        assertEquals(24 * 7 * 2 + 3, listLastTwoWeeks().size());

        SecurityUtil.setUsername(TEST_ADMIN);

        assertEquals(24 + 3, listLastDay().size());
    }
}