import React, { createContext, useState, useEffect } from 'react'

import axios from 'axios'

const ApplicationContext = createContext()

export function ApplicationContextProvider(props) {

    // DEFAULTS

    const defaultUser = {
        authenticated: false,
        permissions: {}
    }

    // STATE HOLDERS

    const [showMenu, setShowMenu] = useState(false)
    const [currentUser, setCurrentUserX] = useState(defaultUser)
    const [myFacility, setMyFacility] = useState({})
    const [facilities, setFacilities] = useState([])
    const [facilityTypes, setFacilityTypes] = useState([])
    const [facilityDistricts, setFacilityDistricts] = useState([])
    const [users, setUsers] = useState([])
    const [counterUpdates, setCounterUpdates] = useState([])
    const [chatConversations, setChatConversations] = useState([])
    const [chatMessagesCount, setChatMessagesCount] = useState(null)
    const [outreachRecords, setOutreachRecords] = useState([])
    const [organizations, setOrganizations] = useState([])

    const setCurrentUser = (u) => {
        setCurrentUserX(u)
    }

    // APPLICATION METHODS

    const extractMessageCode = (error) => {
        if (error.response) {
            if (error.response.data && error.response.data.code) {
                return error.response.data.code
            } else {
                return error.response.status
            }
        } else if (error.request) {
            return "REQUEST_ERROR"
        }

        console.log("Error: " + error.message)
        return "ERROR"
    }

    const toggleMenu = () => {
        setShowMenu((showMenu) => !showMenu)
    }

    const loginUser = (form, onSuccess, onError) => {
        axios
            .post('/api/authentication/login', form)
            .then(r => {
                onSuccess()
                setCurrentUser(r.data)
            })
            .catch(e => {
                onError(extractMessageCode(e))
                setCurrentUser({
                    ...defaultUser,
                    error: "Chyba odeslání: " + e.message
                })
            })
            .then(() => {
                fetchMyFacility()
            })
    }

    const logoutUser = (onSuccess, onError) => {
        axios
            .get('/api/authentication/logout')
            .then(r => {
                setCurrentUser(defaultUser)
                onSuccess()
            })
            .catch(e => {
                onError(extractMessageCode(e))
            })
            .then(() => {
                fetchMyFacility()
            })
    }

    const fetchMe = () => {
        axios
            .get('/api/authentication/me')
            .then(r => {
                const newUser = r.data
                if (newUser.username !== currentUser.username) {
                    fetchData()
                }
                setCurrentUser(newUser)
            })
            .catch(e => {
                console.log(e)
            })
            .then(() => {
                fetchMyFacility()
            })
    }

    const fetchFacilities = () => {
        axios
            .get('/api/facility/list')
            .then(r => setFacilities(r.data))
    }

    const fetchMyFacility = () => {
        axios
            .get('/api/facility/my')
            .then(r => {
                setMyFacility(r.data)
            })
            .catch(e => {
                setMyFacility({ error: extractMessageCode(e) })
            })
    }

    const fetchFacilityTypes = () => {
        axios
            .get('/api/facility-type/list')
            .then(r => setFacilityTypes(r.data))
    }

    const fetchFacilityDistricts = () => {
        axios
            .get('/api/facility-district/list')
            .then(r => setFacilityDistricts(r.data))
    }

    const saveFacility = (facility, onSuccess, onError) => {
        axios.request({
            url: '/api/facility/save',
            method: 'post',
            data: facility
        }).then(() => {
            onSuccess()
            fetchFacilities()
        }).catch(e => {
            onError(extractMessageCode(e))
        })
    }

    const deleteFacility = (facility, onSuccess, onError) => {
        axios.request({
            url: '/api/facility/delete',
            method: 'post',
            data: facility
        }).then(() => {
            onSuccess()
            fetchFacilities()
        }).catch(e => {
            onError(extractMessageCode(e))
        })
    }

    const saveFacilityType = (facilityType, onSuccess, onError) => {
        axios.request({
            url: '/api/facility-type/save',
            method: 'post',
            data: facilityType
        }).then(() => {
            onSuccess()
            fetchFacilityTypes()
        }).catch(e => {
            onError(extractMessageCode(e))
        })
    }

    const deleteFacilityType = (facilityType, onSuccess, onError) => {
        axios.request({
            url: '/api/facility-type/delete',
            method: 'post',
            data: facilityType
        }).then(() => {
            onSuccess()
            fetchFacilityTypes()
        }).catch(e => {
            onError(extractMessageCode(e))
        })
    }

    const generateStamp = () => {
        const stamp = Date.now() + Math.random()
        return stamp
    }
    
    const updateOccupancy = (counterId, difference, onSuccess, onError) => {
        const data = {
            id: counterId,
            difference: difference,
            stamp: generateStamp()
        }

        axios.request({
            url: '/api/facility/occupancy',
            method: 'post',
            data: data
        }).then(r => {
            setMyFacility(r.data)
            onSuccess()
        }).catch(e => {
            onError(extractMessageCode(e))
        })
    }

    const setOccupancy = (counterId, occupancy, onSuccess, onError) => {
        const data = {
            id: counterId,
            value: occupancy,
            stamp: generateStamp()
        }

        axios.request({
            url: '/api/facility/occupancy',
            method: 'post',
            data: data
        }).then(r => {
            setMyFacility(r.data)
            onSuccess()
        }).catch(e => {
            onError(extractMessageCode(e))
        })
    }

    const fetchUsers = () => {
        axios
            .get('/api/user/list')
            .then(r => {
                setUsers(r.data)
            })
    }

    const createUser = (user, onSuccess, onError) => {
        axios.request({
            url: '/api/user/create',
            method: 'post',
            data: user
        }).then(r => {
            onSuccess()
            fetchUsers()
        }).catch(e => {
            onError(extractMessageCode(e))
        })
    }

    const updateUser = (user, onSuccess, onError) => {
        axios.request({
            url: '/api/user/update',
            method: 'post',
            data: user
        }).then(r => {
            onSuccess()
            fetchUsers()
        }).catch(e => {
            onError(extractMessageCode(e))
        })
    }

    const deleteUser = (user, onSuccess, onError) => {
        axios.request({
            url: '/api/user/delete',
            method: 'post',
            data: user
        }).then(() => {
            onSuccess()
            fetchUsers()
        }).catch(e => {
            onError(extractMessageCode(e))
        })
    }

    const changePassword = (data, onSuccess, onError) => {
        axios.request({
            url: '/api/user/change-password',
            method: 'post',
            data: data
        }).then(r => {
            onSuccess()
        }).catch(e => {
            onError(extractMessageCode(e))
        })
    }

    const fetchCounterUpdates = () => {
        axios
            .get('/api/notification/counter-update/expired')
            .then(r => setCounterUpdates(r.data))
    }

    const countNotifications = () => {
        return null
    }

    const snoozeCounterUpdate = (counterId, onSuccess, onError) => {
        axios.request({
            url: '/api/notification/counter-update/snooze',
            method: 'post',
            params: {
                counterId: counterId
            }
        }).then(r => {
            onSuccess()
            fetchCounterUpdates()
        }).catch(e => {
            onError(extractMessageCode(e))
        })
    }

    const fetchChatConversations = () => {
        axios
            .get('/api/chat/conversation/list')
            .then(res => {
                setChatConversations(res.data)

                const newCount = res.data
                    .map(c => c.messages.length)
                    .reduce((a, b) => a + b, 0)
                
                setChatMessagesCount(oldCount => {
                    if (oldCount && oldCount != newCount) {
                        chatMessageSound.play()
                    }

                    return newCount
                })
            })
    }
    
    const chatMessageSound = new Audio("/app/chat-incoming-message-1.mp3")

    const getChatConversationById = (id) => {
        return chatConversations
            .find(c => c.id == id)
    }

    const postChatMessage = (message, onSuccess, onError) => {
        axios.request({
            url: '/api/chat/message/answer',
            method: 'post',
            data: message
        }).then(r => {
            onSuccess()
            fetchChatConversations()
        }).catch(e => {
            onError(extractMessageCode(e))
        })
    }

    const closeChatConversation = (conversationId, onSuccess, onError) => {
        axios.request({
            url: '/api/chat/conversation/close',
            method: 'post',
            params: {
                conversationId: conversationId
            }
        }).then(r => {
            onSuccess()
            fetchChatConversations()
        }).catch(e => {
            onError(extractMessageCode(e))
        })
    }

    const countUnreadChats = () => {
        return null
    }

    const fetchOutreachRecords = (all) => {
        const url = all
            && '/api/outreach/record/list-all-latest'
            || '/api/outreach/record/list-my-latest'

        axios
            .get(url)
            .then(r => setOutreachRecords(r.data))
    }

    const submitOutreachRecord = (record, onSuccess, onError) => {
        axios.request({
            url: '/api/outreach/record/create',
            method: 'post',
            data: record
        }).then(r => {
            onSuccess()
        }).catch(e => {
            onError(extractMessageCode(e))
        })
    }

    const getOutreachRecordById = (id) => {
        return outreachRecords
            .find(r => r.id == id)
    }

    const fetchOrganizations = () => {
        axios
            .get('/api/organization/list')
            .then(r => setOrganizations(r.data))
    }

    const fetchData = () => {
        fetchFacilities()
        fetchUsers()
    }

    // ON START

    useEffect(() => {
        fetchMe()

        const meTimer = setInterval(() => fetchMe(), 10 * 1000)
        const dataTimer = setInterval(() => fetchData(), 60 * 1000)

        return () => {
            clearInterval(meTimer)
            clearInterval(dataTimer)
        }
    }, [])

    // RENDER

    return <ApplicationContext.Provider value={{
        showMenu,
        toggleMenu,

        currentUser,
        loginUser,
        logoutUser,

        myFacility,
        fetchMyFacility,
        updateOccupancy,
        setOccupancy,

        facilities,
        fetchFacilities,
        saveFacility,
        deleteFacility,

        facilityTypes,
        fetchFacilityTypes,
        saveFacilityType,
        deleteFacilityType,

        facilityDistricts,
        fetchFacilityDistricts,

        users,
        fetchUsers,
        createUser,
        updateUser,
        deleteUser,
        changePassword,

        counterUpdates,
        fetchCounterUpdates,
        countNotifications,
        snoozeCounterUpdate,

        chatConversations,
        fetchChatConversations,
        getChatConversationById,
        postChatMessage,
        closeChatConversation,
        countUnreadChats,

        outreachRecords,
        fetchOutreachRecords,
        submitOutreachRecord,
        getOutreachRecordById,

        organizations,
        fetchOrganizations
    }}>
        {props.children}
    </ApplicationContext.Provider>
}

export default ApplicationContext