import React, { useState, useContext, Fragment } from 'react'

import MdcHumanMaleFemale from '@meronex/icons/mdc/MdcHumanMaleFemale'
import MdcHandRight from '@meronex/icons/mdc/MdcHandRight'
import FaMoneyBillWave from '@meronex/icons/fa/FaMoneyBillWave'
import IosMail from '@meronex/icons/ios/IosMail'
import AiFillPhone from '@meronex/icons/ai/AiFillPhone'
import EnEmail from '@meronex/icons/en/EnEmail'
import GoGlobe from '@meronex/icons/go/GoGlobe'
import FaRegSnowflake from '@meronex/icons/fa/FaRegSnowflake'

import { useForm, Form, Label, TextInput, TextArea, Submit, Error, RadioInput } from '../components/Form'
import Button from '../components/Button'
import ApplicationContext from '../application/ApplicationContext'
import StyleContext from '../application/StyleContext'
import { Item, Row, ErrorRow, Field, HalfRow, Subtitle, Title, ConditionalRow, FormRow, TitleRow, Warning } from './Item'
import Counter from './Counter'
import { Highlight } from './Filter'

const counterStyle = {
    marginLeft: '0.3em'
}

const countersFieldStyle = {
    textAlign: 'right'
}

function LengthError({form, name, length}) {
    if (!length) {
        length = 255
    }

    return <Error
        form={form}
        condition={data => data[name] && data[name].length > length}
    >
        <span> </span>
        <span>překračuje maximální povolenou délku {length} znaků.</span>
    </Error>
}

function isTime(number) {
    return /^([01]?[0-9]|2[0-3]):[0-5][0-9](:[0-5][0-9])?$/.test(number)
}

function HourError({form, name}) {
    return <Error
        form={form}
        condition={data => data[name] && !isTime(data[name])}
    >
        <span> </span>
        <span>není čas ve formátu HH:MM.</span>
    </Error>
}

function CounterForms({form}) {
    const counterFormStyle = {
        marginTop: '0.5em',
        paddingTop: '0.5em',
        paddingLeft: '0.5em',
        borderTop: '1px solid white',
        borderLeft: '1px solid white'
    }

    let count = 1

    return form.data.counters
        .map((c, i) => {
            if (c.disabled) {
                return null
            }

            const inactive = c.title ? false : true

            if (c.resetAt) {
                const m = c.resetAt.match(/^(.*:.*):.*$/i)
                if (m) {
                    c.resetAt = m[1]
                }
            }

            return <div key={i} style={counterFormStyle}>
                <FormRow>
                    <Label htmlFor={'counters[' + i + '].title'}>Počítadlo č. {count++} (smazáním názvu se skryje)</Label>
                    <TextInput name={'counters[' + i + '].title'} form={form} />
                </FormRow>
                <FormRow left disabled={inactive}>
                    <Label htmlFor={'counters[' + i + '].occupancy'}>Obsazenost</Label>
                    <TextInput name={'counters[' + i + '].occupancy'} form={form} />
                </FormRow>
                <FormRow right disabled={inactive}>
                    <Label htmlFor={'counters[' + i + '].capacity'}>Kapacita</Label>
                    <TextInput name={'counters[' + i + '].capacity'} form={form} />
                </FormRow>
                <FormRow left disabled={inactive}>
                    <Label htmlFor={'counters[' + i + '].resetAt'}>Automaticky vynulovat (HH:MM)</Label>
                    <TextInput name={'counters[' + i + '].resetAt'} form={form} />
                </FormRow>
                <Error form={form} condition={() => {
                    const resetAt = form.getValue('counters[' + i + '].resetAt')
                    return resetAt && !resetAt.match(/^([0-1]?[0-9]|2[0-3]):[0-5][0-9]$/i)
                }}>
                    <HalfRow>
                        <span>Zadejte ve formátu HH:MM.</span>
                    </HalfRow>
                </Error>
                <Error form={form} condition={() => {
                    const capacity = parseInt(form.getValue('counters[' + i + '].capacity'))
                    const occupancy = parseInt(form.getValue('counters[' + i + '].occupancy'))
                    return capacity > 0 && occupancy > capacity
                }}>
                    <HalfRow>
                        <span>Obsazenost překračuje kapacitu.</span>
                    </HalfRow>
                </Error>
            </div>
        })
}

function FacilityEdit(props) {
    const facility = {...props.facility}

    const { facilityDistricts, facilityTypes } = useContext(ApplicationContext)

    facility.counters = facility.counters || [
        {
            'title': 'bez rozlišení'
        },
        {
            'title': null
        }
    ]

    if (facility.expirationDecisiveTime) {
        const m = facility.expirationDecisiveTime.match(/^(.*:.*):.*$/i)
        if (m) {
            facility.expirationDecisiveTime = m[1]
        }
    }

    if (facility.expirationNotificationTime) {
        const m = facility.expirationNotificationTime.match(/^(.*:.*):.*$/i)
        if (m) {
            facility.expirationNotificationTime = m[1]
        }
    }

    const { form } = useForm(facility, { showErrors: true })

    const errorMessages = {
        'SECURITY_ERROR': 'Nemáte práva editovat toto zařízení.'
    }

    const districtDivStyle = {
        marginTop: '4px'
    }

    return <Item>
        <Form form={form} onSubmit={props.onSubmit}>

            <FormRow>
                <Label htmlFor="name">Název<LengthError name="name" form={form}/></Label>
                <TextInput name="name" form={form}/>
            </FormRow>
            <Error form={form} condition={() => !form.getValue("name")}>
                <HalfRow>
                    <span>Název musí být vyplněn.</span>
                </HalfRow>
            </Error>

            <FormRow>
                <Label htmlFor="organization">Organizace<LengthError name="organization" form={form}/></Label>
                <TextInput name="organization" form={form}/>
            </FormRow>

            <FormRow>
                <Label htmlFor="address">Adresa<LengthError name="address" form={form}/></Label>
                <TextInput name="address" form={form}/>
            </FormRow>

            <FormRow>
                <Label htmlFor="phone">Telefon<LengthError name="phone" form={form}/></Label>
                <TextInput name="phone" form={form}/>
            </FormRow>

            <FormRow>
                <Label htmlFor="email">E-mail<LengthError name="email" form={form}/></Label>
                <TextInput name="email" form={form}/>
            </FormRow>

            <FormRow>
                <Label htmlFor="url">Web<LengthError name="url" form={form}/></Label>
                <TextInput name="url" form={form}/>
            </FormRow>

            <FormRow>
                <Label>Městská část</Label>
                <div style={{display: 'flex', flexWrap: 'wrap', width: '100%'}}>
                    <div style={{width: '50%'}}>
                        {
                            facilityDistricts.filter(d => d.name.startsWith('Praha')).map((d, i) => <div style={districtDivStyle} key={i}>
                                <RadioInput name="facilityDistrict.id" value={d.id} form={form}>{d.name}</RadioInput>
                            </div>)
                        }
                    </div>
                    <div style={{width: '50%'}}>
                    {
                            facilityDistricts.filter(d => !d.name.startsWith('Praha')).map((d, i) => <div style={districtDivStyle} key={i}>
                                <RadioInput name="facilityDistrict.id" value={d.id} form={form}>{d.name}</RadioInput>
                            </div>)
                        }
                    </div>
                </div>
            </FormRow>

            <FormRow>
                <Label>Kategorie zařízení</Label>
                {
                    facilityTypes.filter(t => !t.deleted).map((t, i) => (
                        <RadioInput name="facilityType.id" value={t.id} form={form} key={i}>{t.name}</RadioInput>
                    ))
                }
            </FormRow>

            <FormRow>
                <Label>Zimní opatření?</Label>
                <RadioInput name="winter" value={true} form={form}>ano</RadioInput>
                <RadioInput name="winter" value={false} form={form}>ne</RadioInput>
            </FormRow>

            <FormRow>
                <Label>Povinnost zadat kapacitu</Label>
            </FormRow>
            <FormRow left disabled={!form.data['winter']}>
                <Label htmlFor="expirationDecisiveTime">Nejdříve od (ve formátu HH:MM)<HourError name="expirationDecisiveTime" form={form}/></Label>
                <TextInput name="expirationDecisiveTime" form={form}/>
            </FormRow>
            <FormRow right disabled={!form.data['winter']}>
                <Label htmlFor="expirationNotificationTime">Nejpozději do (ve formátu HH:MM)<HourError name="expirationNotificationTime" form={form}/></Label>
                <TextInput name="expirationNotificationTime" form={form}/>
            </FormRow>

            <CounterForms form={form}/>

            <FormRow>
                <Label htmlFor="targetGroup">Cílová skupina</Label>
                <RadioInput name="targetGroup" value="bez omezení" form={form}>bez omezení</RadioInput>
                <RadioInput name="targetGroup" value="muži" form={form}>muži</RadioInput>
                <RadioInput name="targetGroup" value="ženy" form={form}>ženy</RadioInput>
                <TextInput name="targetGroup" form={form}/>
            </FormRow>

            <FormRow>
                <Label htmlFor="restrictions">Omezení</Label>
                <RadioInput name="restrictions" value="bez omezení" form={form}>bez omezení</RadioInput>
                <RadioInput name="restrictions" value="zákaz alkoholu" form={form}>zákaz alkoholu</RadioInput>
                <TextInput name="restrictions" form={form}/>
            </FormRow>

            <FormRow left>
                <Label htmlFor="fee">Poplatek</Label>
                <TextInput name="fee" form={form}/>
            </FormRow>

            <FormRow>
                <Label htmlFor="note">Poznámka<LengthError form={form} name="note" length="1024"/></Label>
                <TextArea name="note" form={form}/>
            </FormRow>

            <ErrorRow messages={errorMessages} error={props.error} />

            <Row>
                <Field>
                    <Button label="Zrušit" action={props.onCancelClicked} />
                    <Submit label="Uložit" form={form} loading={props.loading} />
                </Field>
            </Row>

        </Form>
    </Item>
}

function FacilityDelete(props) {
    return <Item>
        <TitleRow>
            <Field>
                <Winter facility={props.facility}/>
                <Title>{props.facility.name}</Title>
            </Field>
        </TitleRow>
        <FormRow>
            <Warning>Opravdu smazat?</Warning>
            <Button label="Opravdu smazat" action={() => props.onDeleteClicked(props.facility)} />
        </FormRow>

        <ErrorRow error={props.error}/>

        <Row>
            <Field>
                <Button label="Zrušit" action={props.onCancelClicked} />
            </Field>
        </Row>
    </Item>
}

function FacilityPlus(props) {
    return <Item {...props}>
        <Button label="+" action={props.onEditClicked} />
    </Item>
}

function Winter(props) {
    if (!props.facility || !props.facility.winter) {
        return null
    }

    return <span style={{paddingRight: '.5em'}}>
        <FaRegSnowflake/>
    </span>
}

function Counters({facility}) {
    return facility.counters
        .filter(c => !c.disabled && c.title)
        .map((c, i) => <Counter counter={c} style={counterStyle} key={i} />)
}

function FacilityDetail(props) {
    const { currentUser, myFacility } = useContext(ApplicationContext)
    const { itemLinkStyle } = useContext(StyleContext)

    const facility = props.facility

    const canEdit = currentUser.permissions.UPDATE_ANY_FACILITY || (
        currentUser.permissions.UPDATE_OWN_FACILITY && props.facility.id === myFacility.id
    )

    const canDelete = currentUser.permissions.DELETE_FACILITY && props.facility.id != myFacility.id

    const editButtonRow = <Row>
        <Field>
            <Button label="Upravit" action={props.onEditClicked} enabled={canEdit}/>
            <Button label="Smazat" action={props.onDeleteClicked} enabled={canDelete}/>
        </Field>
    </Row>

    return <Item>
        <TitleRow onClick={props.onItemClicked}>
            <Field>
                <Winter facility={facility}/>
                <Title>
                    <Highlight filter={props.filter}>{facility.name}</Highlight>
                </Title>
            </Field>
            <Field style={countersFieldStyle}>
                <Counters facility={facility}/>
            </Field>
        </TitleRow>
        <Row>
            <Subtitle>
                <Highlight filter={props.filter}>{facility.organization}</Highlight>
            </Subtitle>
        </Row>
        <ConditionalRow icon={IosMail}>
            <Highlight filter={props.filter}>{facility.address}</Highlight>
        </ConditionalRow>
        <ConditionalRow icon={AiFillPhone}>{facility.phone}</ConditionalRow>
        <ConditionalRow icon={EnEmail}>{facility.email}</ConditionalRow>
        <ConditionalRow icon={GoGlobe} condition={facility.url} scrollable>
            <a href={facility.url} target="_blank" style={itemLinkStyle}>{facility.url}</a>
        </ConditionalRow>
        <ConditionalRow icon={MdcHumanMaleFemale}>{facility.targetGroup}</ConditionalRow>
        <ConditionalRow icon={MdcHandRight}>{facility.restrictions}</ConditionalRow>
        <ConditionalRow icon={FaMoneyBillWave}>{facility.fee}</ConditionalRow>
        <ConditionalRow>{facility.note}</ConditionalRow>
        {
            props.editable && editButtonRow
        }
    </Item>
}

function DeletedFacility(props) {
    const facility = props.facility

    return <Item deleted>
        <Row>
            <Field>
                <Winter facility={facility}/>
                <Subtitle>{facility.name}</Subtitle>
            </Field>
        </Row>
    </Item>
}

function FacilityPreview(props) {
    const facility = props.facility

    return <Item onClick={props.onItemClicked}>
        <Row>
            <Field>
                <Winter facility={facility}/>
                <Subtitle><Highlight filter={props.filter}>{facility.name}</Highlight></Subtitle>
            </Field>
            <Field style={countersFieldStyle}>
                <Counters facility={facility}/>
            </Field>
        </Row>
    </Item>
}

export default function Facility(props) {
    const { currentUser, saveFacility, deleteFacility } = useContext(ApplicationContext)

    const [view, setView] = useState("preview")

    const [loading, setLoading] = useState(false)
    const [error, setError] = useState(null)

    const handleSuccess = () => {
        setError(null)
        setLoading(false)
        setView("detail")
    }

    const handleError = (e) => {
        setLoading(false)
        setError(e)
    }

    const handleSubmit = (facility) => {
        setLoading(true)
        saveFacility(facility, handleSuccess, handleError)
    }

    const handleDelete = (facility) => {
        deleteFacility(facility, handleSuccess, handleError)
    }

    const autoDetail = props.facility && props.filter && props.filter.getQuery() && props.filter.anyMatch([
        props.facility.organization,
        props.facility.address
    ]);

    if (view == "edit") {
        return <FacilityEdit
            {...props}
            onCancelClicked={() => setView("detail")}
            onSubmit={handleSubmit}
            loading={loading}
            error={error}
        />
    } else if (props.facility && view == "detail" || autoDetail) {
        return <FacilityDetail
            {...props}
            onItemClicked={() => setView("preview")}
            onEditClicked={() => setView("edit")}
            onDeleteClicked={() => setView("delete")}
        />
    } else if (props.facility && view == "delete") {
        return <FacilityDelete
            {...props}
            onCancelClicked={() => setView("detail")}
            onDeleteClicked={handleDelete}
            loading={loading}
            error={error}
        />
    } else if (props.facility && !props.facility.deleted) {
        return <FacilityPreview
            {...props}
            onItemClicked={() => setView("detail")}
        />
    } else if (props.facility) {
        return <DeletedFacility
            {...props}
        />
    } else if (currentUser.permissions.CREATE_FACILITY) {
        return <FacilityPlus
            {...props}
            onEditClicked={() => setView("edit")}
        />
    } else {
        return null
    }
}
