import React, { useContext, useState } from "react"
import ApplicationContext from "../application/ApplicationContext"
import StyleContext from "../application/StyleContext"
import Button from "./Button"
import { ConditionalRow, ErrorRow, Field, FormRow, Row, Title, Warning } from "./Item"
import GoListOrdered from '@meronex/icons/go/GoListOrdered'
import MdTitle from '@meronex/icons/md/MdTitle'
import { Error, Form, Label, Submit, TextInput, useForm } from "./Form"
import { Highlight } from "./Filter"

function FacilityTypePreview(props) {
    const { facilityTypeTitleStyle } = useContext(StyleContext)

    const type = props.type || { name: '+' }

    return <>
        {
            type.name &&
            <Title onClick={props.onTitleClicked} style={facilityTypeTitleStyle}>
                <Highlight filter={props.filter}>{type.name}</Highlight>
            </Title>
        }
    </>    
}

function FacilityTypeDetail(props) {
    const { currentUser } = useContext(ApplicationContext)
    const { facilityTypeTitleStyle, facilityTypeEditStyle } = useContext(StyleContext)

    const canEdit = currentUser.permissions.UPDATE_FACILITY_TYPE
    const canDelete = currentUser.permissions.DELETE_FACILITY_TYPE

    const type = props.type || { name: '+', order: '' }

    const editButtonRow = <>
        <Row>
            <Field>
                <Button label="Upravit" action={props.onEditClicked} enabled={canEdit}/>
                <Button label="Smazat" action={props.onDeleteClicked} enabled={canDelete}/>
            </Field>
        </Row>
    </>
    
    return <>
        <div style={facilityTypeEditStyle}>
            <Title onClick={props.onTitleClicked} style={facilityTypeTitleStyle}>
                <Highlight filter={props.filter}>{type.name}</Highlight>
            </Title>
            <ConditionalRow icon={MdTitle}>{type.name}</ConditionalRow>
            <ConditionalRow icon={GoListOrdered}>{type.position}</ConditionalRow>
            {editButtonRow}
        </div>
    </>
}

function FacilityTypeEdit(props) {
    const { facilityTypeEditStyle } = useContext(StyleContext)

    const errorMessages = {
        '502': 'Server nestihl odpovědět ve stanoveném čase.'
    }

    const type = props.type || { name: '', order: '' }
    const { form } = useForm(type)

    form.data.position = parseInt(form.data.position)

    return <>
        <Form form={form} onSubmit={props.onSubmit} style={facilityTypeEditStyle}>
            <FormRow>
                <Label htmlFor="name">Název</Label>
                <TextInput name="name" form={form}/>
            </FormRow>
            <Error form={form} condition={() => !form.getValue('name')}>
                <Row>Vyplťe název kategorie.</Row>
            </Error>

            <FormRow>
                <Label htmlFor="name">Pořadí</Label>
                <TextInput name="position" form={form}/>
            </FormRow>
            <Error form={form} condition={() => !Number.isInteger(form.getValue('position'))}>
                <Row>Pořadí musí být vyjádřené kladným celým číslem.</Row>
            </Error>

            <ErrorRow messages={errorMessages} error={props.error} />

            <Row>
                <Field>
                    <Button label="Zrušit" action={props.onCancelClicked} />
                    <Submit label="Uložit" form={form} loading={props.loading} />
                </Field>
            </Row>
        </Form>
    </>
}

function FacilityTypeDelete(props) {
    const { facilityTypeEditStyle, facilityTypeTitleStyle } = useContext(StyleContext)

    return <>
        <div style={facilityTypeEditStyle}>
            <Title onClick={props.onTitleClicked} style={facilityTypeTitleStyle}>{props.type.name}</Title>
            <FormRow>
                <Warning>Opravdu smazat?</Warning>
                <Button label="Opravdu smazat" action={() => props.onDeleteClicked(props.type)} />
            </FormRow>

            <ErrorRow error={props.error}/>

            <Row>
                <Field>
                    <Button label="Zrušit" action={props.onCancelClicked} />
                </Field>
            </Row>
        </div>
    </>
}

function FacilityTypeDeleted(props) {
    const { facilityTypeEditStyle, facilityTypeDeletedTitleStyle } = useContext(StyleContext)
    
    return <>
        <div style={facilityTypeEditStyle}>
            <Title style={facilityTypeDeletedTitleStyle}>{props.type.name}</Title>
        </div>
    </>
}

export default function FacilityType(props) {
    const { saveFacilityType, deleteFacilityType } = useContext(ApplicationContext)

    const [view, setView] = useState("preview")
    
    const [loading, setLoading] = useState(false)
    const [error, setError] = useState(null)

    const handleSuccess = () => {
        setError(null)
        setLoading(false)
        setView("preview")
    }

    const handleError = (e) => {
        setLoading(false)
        setError(e)
    }

    const handleSubmit = (facilityType) => {
        setLoading(true)
        saveFacilityType(facilityType, handleSuccess, handleError)
    }

    const handleDelete = (facilityType) => {
        setLoading(true)
        deleteFacilityType(facilityType, handleSuccess, handleError)
    }

    if (props.type && props.type.deleted) {
        return <FacilityTypeDeleted {...props} />
    }

    return <>
        {
            view == "preview" &&
            <FacilityTypePreview
                {...props}
                onTitleClicked={() =>  setView(props.type ? "detail" : "edit")}
            />
        }
        {
            view == "detail" &&
            <FacilityTypeDetail
                {...props}
                onTitleClicked={() => setView("preview")}
                onEditClicked={() => setView("edit")}
                onDeleteClicked={() => setView("delete")}
            />
        }
        {
            view == "edit" &&
            <FacilityTypeEdit
                {...props}
                onCancelClicked={() => setView(props.type ? "detail" : "preview")}
                onSubmit={handleSubmit}
                loading={loading}
                error={error}
            />
        }
        {
            view == "delete" &&
            <FacilityTypeDelete
                {...props}
                onCancelClicked={() => setView("detail")}
                onDeleteClicked={handleDelete}
                loading={loading}
                error={error}
            />
        }
    </>
}