import React, { useContext, useEffect, useState } from 'react'
import sortBy from 'sort-by'
import AiFillEyeInvisible from '@meronex/icons/ai/AiFillEyeInvisible'
import AiFillEye from '@meronex/icons/ai/AiFillEye'
import ImCheckboxChecked from '../icons/ImCheckboxChecked'
import ImCheckboxUnchecked from '../icons/ImCheckboxUnchecked'
import StyleContext from '../application/StyleContext'
import Button from './Button'

export function useForm(initialValues, config) {
    config = {
        checkboxChecked: <ImCheckboxChecked/>,
        checkboxUnchecked: <ImCheckboxUnchecked/>,
        ...config
    }

    const initialData = {...initialValues}
    const [data, setData] = useState(initialData)

    const initialShowErrors = config.showErrors || false
    const [showErrors, setShowErrors] = useState(initialShowErrors)

    const initialHasErrors = false
    const [hasErrors, setHasErrors] = useState(initialHasErrors)

    const [focus, setFocus] = useState(null)

    const clearForm = (newData) => {
        setData({...initialData, ...newData})
        setShowErrors(initialShowErrors)
        setHasErrors(initialHasErrors)
    }

    const getValue = (name) => {
        return getPath(form.data, name) || ''
    }

    const handleChange = (name, value) => {
        const dataCopy = {...data}
        updatePath(dataCopy, name, value)
        setData(dataCopy)
        setHasErrors(false)
    }

    const handleToggleChange = (name, value) => {
        var values
        if (data[name]) {
            values = data[name].includes(value)
                ? data[name].filter(v => v !== value)
                : [...data[name], value]
        } else {
            values = [value]
        }

        setData({...data, [name]: values})
        setHasErrors(false)
    }

    const handleFocusChange = (name) => {
        setFocus(name)
    }

    const handleKeyDown = (name, key) => {
        if (key == 'Escape' || key == 27) {
            handleChange(name, initialData[name])
        }
    }

    const handleError = (hasError) => {
        if (hasError) {
            setHasErrors(true)
        }
    }

    const handleSubmit = () => {
        if (form.hasErrors) {
            setShowErrors(true)
        } else {
            form.onSubmit(form.data)
        }
    }

    const form = {
        data,
        config,
        showErrors,
        hasErrors,
        focus,
        getValue,
        handleChange,
        handleToggleChange,
        handleFocusChange,
        handleKeyDown,
        handleSubmit,
        handleError,
        clearForm,
        onSubmit: null
    }

    return { form }
}

function updatePath(data, path, value) {
    const indices = path
        .split(/[\.\[\]\'\"]/)
        .filter(p => p)

    const penultimate = indices
        .slice(0, -1)
        .reduce((ptr, index) => {
            if (!ptr[index]) {
                ptr[index] = {}
            }
            return ptr[index]
        }, data)

    const ultimate = indices.slice(-1)

    penultimate[ultimate] = value
}

function getPath(data, path) {
    return path
        .split(/[\.\[\]\'\"]/)
        .filter(p => p)
        .reduce((o, p) => o[p], data)
}

function checkFormProp(props) {
    if (!props.form) {
        throw "Form from useForm() hook was not set as a prop."
    }
    if (props.form['form']) {
        throw "Use const { form } = useForm() instead of const form = useForm()."
    }
}

function checkNameProp(props) {
    if (!props.name) {
        throw "Name prop was not set."
    }
}

function checkCondition(props) {
    if (!props.condition) {
        throw "Condition prop was not set."
    }
}

export function Form(props) {
    checkFormProp(props)

    props.form.onSubmit = props.onSubmit

    return <form style={props.style}>
        {props.children}
    </form>
}

function Input(props) {
    checkFormProp(props)
    checkNameProp(props)

    const { inputStyle } = useContext(StyleContext)

    return <input
        id={props.name}
        name={props.name}
        type={props.type}
        style={{...inputStyle, ...props.style}}
        onChange={e => props.form.handleChange(e.target.name, e.target.value)}
        onFocus={e => props.form.handleFocusChange(e.target.name)}
        onBlur={e => props.form.handleFocusChange(null)}
        onKeyDown={e => props.form.handleKeyDown(e.target.name, e.key || e.keyCode)}
        value={props.form.getValue(props.name)}
        autoComplete={props.autoComplete}
    />
}

export function TextInput(props) {
    return <Input {...props} type='text' />
}

export function RadioInput(props) {
    checkFormProp(props)
    checkNameProp(props)

    const {radioLabelStyle} = useContext(StyleContext)

    let value = null
    try {
        value = props.form.getValue(props.name)
    } catch {
        console.log("warning, form subobject is null")
    }

    return <Label>
        <input
            name={props.name}
            value={props.value}
            type='radio'
            style={{display: 'none'}}
            onChange={() => props.form.handleChange(props.name, props.value)}
        />
        {
            value == props.value ? props.form.config.checkboxChecked : props.form.config.checkboxUnchecked
        }
        <span style={radioLabelStyle}>{props.children}</span>
    </Label>
}

export function CheckboxInput(props) {
    checkFormProp(props)
    checkNameProp(props)

    const {radioLabelStyle} = useContext(StyleContext)

    const value = props.form.getValue(props.name)

    return <Label>
        <input
            name={props.name}
            value={props.value}
            type='checkbox'
            style={{display: 'none'}}
            onChange={() => props.form.handleToggleChange(props.name, props.value)}

        />
        {
            value && value.includes(props.value)
                ? props.form.config.checkboxChecked
                : props.form.config.checkboxUnchecked
        }
        <span style={radioLabelStyle}>{props.children}</span>
    </Label>
}

export function PasswordInput(props) {
    const [visible, setVisible] = useState(false)

    const wrapperStyle = {
        width: '100%'
    }

    const eyeStyle = {
        float: 'right',
        color: 'black',
        position: 'relative',
        bottom: '1.6em',
        right: '0.5em'
    }

    return <div style={wrapperStyle}>
        <Input {...props} type={visible ? 'text' : 'password'} />
        <span onClick={() => setVisible(v => !v)} style={eyeStyle}>
            {
                visible ? <AiFillEyeInvisible /> : <AiFillEye />
            }
        </span>
    </div>
}

export function TextArea(props) {
    checkFormProp(props)
    checkNameProp(props)

    const { inputStyle } = useContext(StyleContext)

    return <textarea
        id={props.name}
        name={props.name}
        style={{...inputStyle, ...props.style}}
        onChange={e => props.form.handleChange(e.target.name, e.target.value)}
        value={props.form.getValue(props.name)}
    />
}

export function Select(props) {
    const { inputStyle } = useContext(StyleContext)

    return <select
        onChange={e => props.form.handleChange(props.name, e.target.value)}
        style={{...inputStyle, ...props.style}}
        value={props.form.getValue(props.name)}
    >
        {props.children}
    </select>
}

export function Option(props) {
    return <option value={props.value}>{props.children}</option>
}

export function Options(props) {
    const items = [...props.items];

    if ('order' in props) {
        const fields = props['order'].split(' ')
        const comparator = sortBy.apply(null, fields)
        items.sort(comparator)
    }

    return items.map(
        (item, i) => <Option key={i} value={props.value(item)}>
            {props.label(item)}
        </Option>
    )
}

export function Label(props) {
    const { labelStyle } = useContext(StyleContext)

    return <label
        htmlFor={props.htmlFor}
        style={{...labelStyle, ...props.style}}
    >
        {props.children}
    </label>
}

export function Submit(props) {
    checkFormProp(props)

    return <Button
        {...props}
        action={props.form.handleSubmit}
        type="submit"
    />
}

export function Error(props) {
    checkFormProp(props)
    checkCondition(props)

    const { errorStyle } = useContext(StyleContext)

    const hasError = props.condition(props.form.data)
    useEffect(() => props.form.handleError(hasError))
    
    if (hasError) {
        return <span style={{...errorStyle, ...props.style}}>
            {props.form.showErrors && props.children}
        </span>
    } else {
        return null
    }
}

function HintItem({input, item, onClick}) {
    let result = []
    let from = 0
    let to = 0

    let k = 0
    while (from < item.length) {
        to = input.length > 0
            ? item.toLowerCase().indexOf(input, from)
            : -1

        if (to !== -1) {
            result.push(<span key={k++}>{item.substring(from, to)}</span>)
            result.push(<mark key={k++}>{item.substring(to, to + input.length)}</mark>)
    
            from = to + input.length
        } else {
            result.push(<span key={k++}>{item.substring(from)}</span>)
            break
        }
    }

    const [hover, setHover] = useState(false)

    const liStyle = {
        padding: '1vmin',
        fontWeight: hover ? '300' : '100'
    }

    return <li
        style={liStyle}
        onMouseEnter={() => setHover(true)}
        onMouseLeave={() => setHover(false)}
        onClick={() => onClick(item)}>{result}</li>
}

export function Hint({input, list, condition, onClick}) {
    input = input || ""
    input = input.toLowerCase()

    list = list || []
    list = list
        .filter(i => i)
        .filter(i => i.toLowerCase().indexOf(input) !== -1)

    const ulStyle = {
        position: 'absolute',
        top: '0',
        background: 'black',
        zIndex: '10',
        marginTop: '0',
        borderRadius: '1vmin',
        paddingLeft: '0',
        listStyle: 'none',
        width: '100%'
    }

    return (
        condition &&
        <ul style={ulStyle}>
            {
                list.map((i, k) => <HintItem input={input} item={i} key={k} onClick={onClick}/>)
            }
        </ul>
    )
}