import React, { useContext } from 'react'
import MessageContext from '../application/MessageContext'
import StyleContext from '../application/StyleContext'
import Loader from './Loader'

export function Item(props) {
    const { itemStyle, itemDeletedStyle } = useContext(StyleContext)

    let style = {...itemStyle, ...props.style}

    if ('deleted' in props) {
        style = {...style, ...itemDeletedStyle}
    }

    return <div onClick={props.onClick} style={style} className={props.className}>
        {props.children}
    </div>
}

export function Title(props) {
    const { itemTitleStyle } = useContext(StyleContext)

    return <span {...props} style={{...itemTitleStyle, ...props.style}}>
        {props.children}
    </span>
}

export function Subtitle(props) {
    const { itemSubtitleStyle } = useContext(StyleContext)

    return <span style={{...itemSubtitleStyle, ...props.style}}>
        {props.children}
    </span>
}

export function Field(props) {
    const { itemFieldStyle } = useContext(StyleContext)

    return <div style={{...itemFieldStyle, ...props.style}}>
        {props.children}
    </div>
}

export function Row(props) {
    const { itemRowStyle, inactiveStyle } = useContext(StyleContext)

    const disabledStyle = props.disabled ? inactiveStyle : {}

    return <div onClick={props.onClick} style={{...itemRowStyle, ...disabledStyle, ...props.style}}>
        {props.children}
    </div>
}

export function HalfRow(props) {
    const { halfRowStyle } = useContext(StyleContext)

    const paddingStyle = {}
    if (props.left) {
        paddingStyle['paddingRight'] = '0.5em'
    }
    if (props.right) {
        paddingStyle['paddingLeft'] = '0.5em'
    }

    return <div style={{...halfRowStyle, ...paddingStyle, ...props.style}}>
        {props.children}
    </div>
}

export function TitleRow(props) {
    const { itemTitleRowStyle } = useContext(StyleContext)

    return <Row {...props} style={{...itemTitleRowStyle, ...props.style}}>
        {props.children}
    </Row>
}

export function FormRow(props) {
    const filteredChildren = React.Children.toArray(props.children).filter(Boolean)

    const isSecondRowRadioInput = filteredChildren.length > 2 && filteredChildren[1].type.name == 'RadioInput'

    let firstStyle;
    if (isSecondRowRadioInput) {
        firstStyle = {}
    } else {
        firstStyle = {
            paddingBottom: 0
        }
    }

    const wrappedChildren = React.Children.map(filteredChildren, (Child, i) =>
        <Row key={i} style={i == 0 && firstStyle} disabled={props.disabled}>{Child}</Row>
    )

    if ("left" in props) {
        return <HalfRow left>{wrappedChildren}</HalfRow>
    } else if ("right" in props) {
        return <HalfRow right>{wrappedChildren}</HalfRow>
    } else {
        return <>{wrappedChildren}</>
    }
}

export function ErrorRow(props) {
    const { errorMessages } = useContext(MessageContext)
    const { errorStyle } = useContext(StyleContext)

    const messages = {
        ...errorMessages,
        ...props.messages
    }

    if (props.error) {
        return <Row style={errorStyle}>
            {messages[props.error] || 'Chyba: ' + props.error}
        </Row>
    } else {
        return null
    }
}

export function ItemLoader(props) {
    const { itemLoaderStyle} = useContext(StyleContext)

    if (props.loading) {
        return <Row style={itemLoaderStyle}>
            <Loader loading={props.loading}/>
        </Row>
    } else {
        return null
    }
}

export function ConditionalRow(props) {
    const Icon = props.icon
    const show = "condition" in props ? props.condition : props.children

    const { itemRowIconStyle } = useContext(StyleContext)

    const rowStyle = {
        ...props.style,
        overflow: 'scrollable' in props ? 'scroll' : 'hidden'
    }

    return <>
    {
        show && <Row style={rowStyle}>
            <Field>
                {"icon" in props && <sup style={itemRowIconStyle}><Icon/></sup>}
                {props.children}
            </Field>
        </Row>
    }
    </>
}

export function Warning(props) {
    const { itemWarningStyle } = useContext(StyleContext)

    return <span style={{...itemWarningStyle, ...props.style}}>
        {props.children}
    </span>
}

export function Center(props) {
    const defaultStyle = {
        textAlign: 'center'
    }

    return <div style={{...defaultStyle, ...props.style}}>
        {props.children}
    </div>
}

export function Separator(props) {
    const defaultStyle = {
        margin: '1em 0',
        borderTop: '1px solid white'
    }

    return <div style={{...defaultStyle, ...props.style}}></div>
}