import React, { useState, useContext, useEffect } from 'react'

import { useForm, Form, Label, Submit, PasswordInput, TextInput, Error, Select, Option, Options, CheckboxInput, Hint } from '../components/Form'
import Button from '../components/Button'
import ApplicationContext from '../application/ApplicationContext'
import { ErrorRow, Field, FormRow, Item, Row, Title, Warning, Separator, HalfRow } from './Item'

const errorMessages = {
    'WRONG_CREDENTIALS': 'Zadali jste špatné heslo.',
    'NEW_PASSWORD_WRONG': 'Nové heslo není validní.'
}

function Explanation(props) {
    const explanationStyle = {
        position: 'relative',
        left: '16px',
        marginLeft: '2vmin',
        fontSize: '0.8em',
        fontStyle: 'italic'
    }
    return <div style={explanationStyle}>{props.children}</div>
}

function UserEdit(props) {
    const user = {...props.user}
    const { form } = useForm(user)
    const { facilities, organizations, fetchOrganizations } = useContext(ApplicationContext)

    const filteredFacilities = facilities.filter(f => !f.deleted)

    useEffect(() => fetchOrganizations(), [])

    const [showOrganizationHint, setShowOrganizationHint] = useState(false)
    useEffect(() => {
        if (form.focus == 'organization') {
            setShowOrganizationHint(true)
        } else if (showOrganizationHint) {
            setTimeout(() => setShowOrganizationHint(false), 300)
        }
    }, [form.focus])
    const handleOrganizationHintClicked = (t) => {
        form.handleChange('organization', t)
        setShowOrganizationHint(false)
    }

    return <Item>
        <Form form={form} onSubmit={props.onSubmit}>
            {
                user.id && <Row>
                    <Title>{user.username}</Title>
                </Row>
            }
            {
                !user.id && <>
                    <FormRow>
                        <Label htmlFor="username">Uživatelské jméno</Label>
                        <TextInput name="username" form={form}/>
                    </FormRow>
                    <Error form={form} condition={() => !form.getValue("username")}>
                        <HalfRow>
                            <span>Uživatelské jméno musí být vyplněno.</span>
                        </HalfRow>
                    </Error>

                    <FormRow>
                        <Label htmlFor="password">Heslo</Label>
                        <TextInput name="password" form={form}/>
                    </FormRow>
                    <Error form={form} condition={() => !form.getValue("password")}>
                        <HalfRow>
                            <span>Heslo musí být vyplněno.</span>
                        </HalfRow>
                    </Error>

                    <Separator />
                </> 
            }

            <FormRow>
                <Label htmlFor="roles">Role</Label>
                <CheckboxInput name="roles" value="FACILITY_OPERATOR" form={form}>operátor zařízení <Explanation>mění obsazenost <strong>svého</strong> zařízení</Explanation></CheckboxInput>
                <CheckboxInput name="roles" value="FACILITY_MANAGER" form={form}>manažer zařízení <Explanation>mění údaje o <strong>svém</strong> zařízení</Explanation></CheckboxInput>
                <CheckboxInput name="roles" value="FACILITY_ADMIN" form={form}>dispečer <Explanation>mění <strong>všechna</strong> zařízení, stahuje reporty, chatuje a vidí terénní šetření</Explanation></CheckboxInput>
                <CheckboxInput name="roles" value="USER_ADMIN" form={form}>administrátor uživatelů <Explanation>mění nastavení <strong>veškerých</strong> uživatelů</Explanation></CheckboxInput>
                <CheckboxInput name="roles" value="OUTREACH_WORKER" form={form}>terénní pracovník <Explanation>vyplňuje <strong>svá</strong> terénní šetření</Explanation></CheckboxInput>
            </FormRow>

            <Separator />
            
            {
                form.data.roles &&
                form.data.roles.includes('OUTREACH_WORKER') &&
                <>
                    <FormRow>
                        <Label htmlFor="name">Celé jméno</Label>
                        <TextInput name="name" form={form}/>
                    </FormRow>

                    <FormRow>
                        <Label htmlFor="organization">Organizace</Label>
                        <TextInput name="organization" form={form} autoComplete="off" />
                        <Hint
                            input={form.data.organization}
                            list={organizations.map(o => o.name)}
                            condition={showOrganizationHint}
                            onClick={handleOrganizationHintClicked}
                        />
                    </FormRow>
                </>
            }

            {
                form.data.roles &&
                (form.data.roles.includes('FACILITY_OPERATOR') || form.data.roles.includes('FACILITY_MANAGER')) &&
                <FormRow>
                    <Label htmlFor="facilityId">Zařízení</Label>
                    <Select name="facilityId" form={form} >
                        <Option value=""></Option>
                        <Options
                            items={filteredFacilities}
                            value={f => f.id}
                            label={f => f.name + (f.organization ? " (" + f.organization + ")" : "")}
                            order="name"
                        />
                    </Select>
                </FormRow>
            }

            <ErrorRow messages={errorMessages} error={props.error} />

            <Row>
                <Field>
                    <Button label="Zrušit" action={props.onCancelClicked} />
                    <Submit label="Uložit" form={form} loading={props.loading} />
                </Field>
            </Row>
        </Form>
    </Item>
}

function UserDelete(props) {
    return <Item>
        <Row>
            <Title>{props.user.username}</Title>
        </Row>
        <FormRow>
            <Warning>Opravdu smazat?</Warning>
            <Button label="Opravdu smazat" action={() => props.onDeleteClicked(props.user)} />
        </FormRow>

        <ErrorRow error={props.error}/>

        <Row>
            <Field>
                <Button label="Zrušit" action={props.onCancelClicked} />
            </Field>
        </Row>
    </Item>
}

function UserChangePassword(props) {
    const { currentUser } = useContext(ApplicationContext)

    const passwordChangeData = {
        username: props.user.username
    }

    const { form } = useForm(passwordChangeData, { showErrors: true })

    return <Item>
        <Form form={form} onSubmit={props.onSubmit}>
            {
                !currentUser.permissions.CHANGE_ANY_PASSWORD && <>
                    <FormRow>
                        <Label htmlFor="currentPassword">Současné heslo</Label>
                        <PasswordInput name="currentPassword" form={form} />
                    </FormRow>
                </>
            }

            <FormRow>
                <Label htmlFor="newPassword">Nové heslo</Label>
                <PasswordInput name="newPassword" form={form} />
            </FormRow>

            <FormRow>
                <Label htmlFor="repeatedPassword">Nové heslo pro kontrolu<Error form={form} condition={({newPassword, repeatedPassword}) => newPassword != repeatedPassword}> se však neshoduje</Error></Label>
                <PasswordInput name="repeatedPassword" form={form} />
            </FormRow>

            <ErrorRow messages={errorMessages} error={props.error} />

            <Row>
                <Field>
                    <Button label="Zrušit" action={props.onCancel} />
                    <Submit label="Změnit" form={form} loading={props.loading} />
                </Field>
            </Row>
        </Form>
    </Item>
}

function UserPlus(props) {
    return <Item>
        <Button label="+" action={props.onEditClicked}/>
    </Item>
}

function UserDetail(props) {
    const { currentUser } = useContext(ApplicationContext)
    const user = props.user

    const canChangePassword = currentUser.permissions.CHANGE_ANY_PASSWORD || (
        currentUser.permissions.CHANGE_OWN_PASSWORD && props.user.username === currentUser.username
    )
    const canUpdate = currentUser.permissions.UPDATE_ANY_USER
    const canDelete = currentUser.permissions.DELETE_USER && props.user.username != currentUser.username

    return <Item>
        <Row><Title>{user.username}</Title></Row>
        <Row>
            <Field>
                <Button label="Změnit heslo" action={props.onPasswordChangeClicked} enabled={canChangePassword}/>
                <Button label="Upravit" action={props.onEditClicked} enabled={canUpdate}/>
                <Button label="Smazat" action={props.onDeleteClicked} enabled={canDelete}/>
            </Field>
        </Row>
    </Item>
}

function DeletedUser(props) {
    return <Item deleted>
        <Row>
            <Field>
                <Title>{props.user.username}</Title>
            </Field>
        </Row>
    </Item>
}

export default function User(props) {
    const { currentUser, createUser, updateUser, changePassword, deleteUser } = useContext(ApplicationContext)

    const [view, setView] = useState(null)

    const [loading, setLoading] = useState(false)
    const [error, setError] = useState(null)

    const handleSuccess = () => {
        setError(null)
        setLoading(false)
        setView(null)
    }

    const handleCancel = handleSuccess

    const handleError = (e) => {
        setLoading(false)
        setError(e)
    }

    const handleCreateUser = (user) => {
        setLoading(true)
        createUser(user, handleSuccess, handleError)
    }

    const handleUpdateUser = (user) => {
        setLoading(true)
        updateUser(user, handleSuccess, handleError)
    }

    const handleChangePassword = (data) => {
        setLoading(true)
        changePassword(data, handleSuccess, handleError)
    }

    const handleDeleteUser = (user) => {
        setLoading(true)
        deleteUser(user, handleSuccess, handleError)
    }

    if (view == "create") {
        return <UserEdit
            {...props}
            onCancelClicked={handleCancel}
            onSubmit={handleCreateUser}
            loading={loading}
            error={error}
        />
    } else if (view == "change-password") {
        return <UserChangePassword
            {...props}
            onSubmit={handleChangePassword}
            onCancel={handleCancel}
            loading={loading}
            error={error}
        />
    } else if (view == "edit") {
        return <UserEdit
            {...props}
            onCancelClicked={handleCancel}
            onSubmit={handleUpdateUser}
            loading={loading}
            error={error}
        />
    } else if (view == "delete") {
        return <UserDelete
            {...props}
            onCancelClicked={handleCancel}
            onDeleteClicked={handleDeleteUser}
            loading={loading}
            error={error}
        />
    } else if (props.user && props.user.deleted) {
        return <DeletedUser
            {...props}
        />
    } else if (props.user) {
        return <UserDetail
            {...props}
            onPasswordChangeClicked={() => setView("change-password")}
            onEditClicked={() => setView("edit")}
            onDeleteClicked={() => setView("delete")}
        />
    } else if (currentUser.permissions.CREATE_USER) {
        return <UserPlus onEditClicked={() => setView("create")} />
    } else {
        return null
    }
}
