import { useContext, useEffect, useState } from 'react'
import { Route, Switch, useHistory, useParams, useRouteMatch } from 'react-router'
import ApplicationContext from '../application/ApplicationContext'
import { ConditionalRow, ErrorRow, Field, FormRow, Item, Row, Title, Warning } from '../components/Item'
import List from '../components/List'
import BsTriangleFill from '@meronex/icons/bs/BsTriangleFill';
import BsSquareFill from '@meronex/icons/bs/BsSquareFill';
import BsCircleFill from '@meronex/icons/bs/BsCircleFill';
import BisTimeFive from '@meronex/icons/bi/BisTimeFive';
import IosSend from '@meronex/icons/ios/IosSend';
import BsCheck from '@meronex/icons/bs/BsCheck';
import { Form, Submit, TextArea, useForm } from '../components/Form'
import Button from '../components/Button'
import StyleContext from '../application/StyleContext'
import useInterval from '../utils/use-interval'

const React = require('react')

const signSymbols = [<BsTriangleFill/>, <BsSquareFill/>, <BsCircleFill/>]
const signColors = ["01295f","437f97","849324","ffb30f","fd151b"]

function Sign(props) {
    const symbol = signSymbols[props.seed % signSymbols.length]
    const color = '#' + signColors[props.seed % signColors.length]

    const style = {
        display: 'inline-flex',
        justifyContent: 'center',
        alignItems: 'center',

        borderRadius: '2px',

        width: '1.2em',
        height: '1.2em',
        padding: '6px 6px 6px 6px',
        margin: '4px',
        marginRight: '1em',

        fontSize: '1.4em',

        backgroundColor: 'white',
        borderColor: color,
        color: color
    }
    
    return <>
        <div style={style}>
            {symbol}
        </div>
    </>
}

function ConversationLink(props) {
    const started = new Date(props.conversation.started)
    const conversationId = props.conversation.id
    const history = useHistory()
    const clickHandler = () => history.push('/chat/' + props.conversation.id)

    const { chatAnsweredStyle, chatClosedStyle } = useContext(StyleContext)
    const itemStyle = props.conversation.closed
        ? chatClosedStyle
        : props.conversation.answered
            ? chatAnsweredStyle
            : {}

    return <>
        <Item onClick={clickHandler} style={itemStyle}>
            <Row>
                <div></div>
                <div>
                    <Sign seed={conversationId}/>
                    <Title style={{color: 'white', textAlign: 'center'}}>{started.toLocaleString()}</Title>
                </div>
                <div></div>
            </Row>
        </Item>
    </>
}

function MessageForm(props) {
    const { postChatMessage, closeChatConversation } = useContext(ApplicationContext)
    const [view, setView] = useState(null)
    const [loading, setLoading] = useState(false)
    const [error, setError] = useState(null)

    const history = useHistory()

    const conversation = props.conversation || {} 

    const { form } = useForm({})

    const handleSuccess = () => {
        setError(null)
        setView(null)
        setLoading(false)

        form.data.messageText = ''
    }

    const handleError = (e) => {
        setLoading(false)
        setView(null)
        setError(e)
    }

    const submit = () => {
        if (!form.data.messageText) {
            return
        }

        setLoading(true)
        postChatMessage({
            conversationId: conversation.id,
            conversationSecret: conversation.secret,
            messageText: form.data.messageText
        }, handleSuccess, handleError)
    }

    const errorMessages = {}

    const { buttonIconStyle } = useContext(StyleContext)

    const submitLabel = <>
        Odeslat
        <sup><IosSend style={buttonIconStyle}/></sup>
    </>

    const closeLabel = <>
        Uzavřít
        <sup><BsCheck style={buttonIconStyle}/></sup>
    </>

    const closeClicked = () => {
        setView("close")
    }

    const cancelClicked = () => {
        setView(null)
    }

    const reallyCloseClicked = () => {
        setLoading(true)
        closeChatConversation(conversation.id, () => {
            handleSuccess()
            history.push('/chat')
        }, handleError)
    }

    if (view == "close") {
        return <>
            <Warning>Opravdu uzavřít?</Warning>
            <Row>
                <Button label="Opravdu uzavřít" action={reallyCloseClicked} loading={loading} />
            </Row>
            <Row>
                <Button label="Zrušit" action={cancelClicked}/>
            </Row>
        </>
    }
    
    return <>
        <Form form={form} onSubmit={submit}>
            <FormRow>
                <TextArea name="messageText" form={form}/>
            </FormRow>
            <ErrorRow messages={errorMessages} error={error} />
            <FormRow>
                <Field>
                    <Submit label={submitLabel} form={form} loading={loading} />
                    <Button label={closeLabel} action={closeClicked} />
                </Field>
            </FormRow>
        </Form>
    </>
}

function ConversationPage(props) {
    const { id } = useParams()
    const { getChatConversationById, fetchChatConversations } = useContext(ApplicationContext)
    
    const conversation = getChatConversationById(id) || {}
    const started = conversation.started && new Date(conversation.started).toLocaleString()
    const messages = conversation.messages || []

    // navigation
    const history = useHistory()
    const clickHandler = () => history.push('/chat')

    useEffect(() => {
        const el = document.getElementsByClassName("chatMessages")[0]
        if (el) {
            el.scrollTop = el.scrollHeight
        }    
    })

    const { chatClosedStyle} = useContext(StyleContext)
    
    let containerStyle = {
        display: 'flex',
        flexDirection: 'column',
        height: '100%',
        ...(conversation.closed ? chatClosedStyle : {})
    }

    const messageListStyle = {
        flex: 1,
        overflow: 'scroll'
    }

    return <>
        <div style={containerStyle}>
            <Item onClick={clickHandler} style={{background: 'none'}}>
                <Row>
                    <div></div>
                    <div>
                        <Sign seed={id}/>
                        <Title>{started}</Title>
                    </div>
                    <div></div>
                </Row>
            </Item>
            <List items={messages} item="message" ulStyle={messageListStyle} ulClassName="chatMessages" liClassName="chatMessage">
                <Message/>
            </List>
            <MessageForm conversation={conversation}/>
        </div>
    </>
}

function Message({message}) {
    const itemStyle = {
        margin: message.user ?  '0 0 1em 4em' : '0 4em 1em 0'
    }

    const textStyle = {
        fontSize: '1.2em'
    }

    const postedStyle = {
        fontSize: '0.8em'
    }

    const text = message.text
    const posted = message.posted && new Date(message.posted).toLocaleString()

    return <>
        <Item style={itemStyle}>
            <ConditionalRow style={textStyle}>{text}</ConditionalRow>
            <ConditionalRow icon={BisTimeFive} style={postedStyle}>{posted}</ConditionalRow>
        </Item>
    </>
}

export default function Chat(props) {
    const { currentUser, chatConversations, fetchChatConversations } = useContext(ApplicationContext)

    useInterval(() => fetchChatConversations())

    const match = useRouteMatch()
    
    if (!currentUser.permissions.REPLY_TO_ANY_CHAT) {
        return null
    }

    return <>
        <Switch>
            <Route path={match.path + "/:id"}>
                <ConversationPage/>
            </Route>
            <Route path={match.path}>
                <List items={chatConversations} item="conversation" order="closed -started">
                    <ConversationLink/>
                </List>
            </Route>
        </Switch>
    </>
}