import React, { useContext, useEffect, useState } from 'react'
import ApplicationContext from '../application/ApplicationContext'
import { Error, Form, Label, Submit, TextArea, TextInput, useForm } from '../components/Form'
import { ErrorRow, Field, FormRow, Item, Row, Subtitle, Title } from '../components/Item'
import List from '../components/List'

export default function OutreachRecords() {
    const { currentUser, fetchOutreachRecords, outreachRecords } = useContext(ApplicationContext)

    const all = currentUser.permissions.VIEW_ALL_OUTREACH_RECORDS ? true : false
    useEffect(() => fetchOutreachRecords(all), [all])

    return <>
        {
            currentUser.permissions.CREATE_OUTREACH_RECORD && <RecordCreate />
        }
        <List items={outreachRecords} item="record">
            <RecordLink />
        </List>
    </>
}

function RecordLink({record}) {
    const [ view, setView ] = useState(null)
    const handleClicked = () => setView("detail")
    const handleClose = () => setView(null)

    const created = new Date(record.created).toLocaleString()

    if (view == 'detail') {
        return <RecordDetail record={record} onClick={handleClose}/>
    }

    return <Item onClick={handleClicked}>
        <Row>
            <Title>{created}</Title>
            <Subtitle>{record.user && record.user.name}</Subtitle>
        </Row>
    </Item>
}

function RecordDetail({record, onClick}) {
    const { fetchOutreachRecords } = useContext(ApplicationContext)
    useEffect(() => fetchOutreachRecords(), [])
    
    const created = new Date(record.created).toLocaleString()
    
    return <Item onClick={onClick}>
        <Row>
            <Title>{created}</Title>
            <Subtitle>{record.user && record.user.name}</Subtitle>
        </Row>
        <RecordRow label="Místo kontaktu" value={record.location}/>
        <RecordRow label="Potřeby klienta/charakteristika" value={record.description}/>
        <RecordRow label="Pseudonym" value={record.nickname}/>
        <RecordRow label="Záznam o řešení" value={record.intervention}/>
        <RecordRow label="Jméno pracovníka" value={record.worker}/>
    </Item>
}

function RecordRow(props) {
    const labelStyle = {
        fontSize: '0.8em',
        marginTop: '1em',
        fontStyle: 'italic'
    }

    return <>
        <Row style={labelStyle}>{props.label}</Row>
        <Row>{props.value}</Row>
    </>
}

function RecordCreate() {
    const { fetchOutreachRecords, submitOutreachRecord } = useContext(ApplicationContext)
    const [loading, setLoading] = useState(false)
    const [error, setError] = useState(null)

    const { form } = useForm()

    const handleSubmit = () => {
        submitOutreachRecord(form.data, handleSuccess, handleError)
        setLoading(true)
    }

    const handleSuccess = () => {
        setLoading(false)
        form.clearForm({ worker: form.data.worker })
        fetchOutreachRecords()
    }

    const handleError = (code) => {
        setLoading(false)
        setError(code)
    }

    const errorMessages = {}

    return <Form form={form} onSubmit={handleSubmit}>
        <FormRow>
            <Label htmlFor="location">Místo kontaktu</Label>
            <TextInput name="location" form={form} />
            <LengthError name="location" form={form} />
        </FormRow>
        <FormRow>
            <Label htmlFor="description">Potřeby klienta/charakteristika</Label>
            <TextArea name="description" form={form} />
            <LengthError name="description" form={form} length="1024" />
        </FormRow>
        <FormRow>
            <Label htmlFor="nickname">Pseudonym</Label>
            <TextInput name="nickname" form={form}/>
            <LengthError name="nickname" form={form} />
        </FormRow>
        <FormRow>
            <Label htmlFor="intervention">Záznam o řešení</Label>
            <TextArea name="intervention" form={form} />
            <LengthError name="intervention" form={form} length="1024" />
        </FormRow>
        <FormRow>
            <Label htmlFor="worker">Jméno pracovníka</Label>
            <TextInput name="worker" form={form} />
            <LengthError name="worker" form={form} />
        </FormRow>
        <ErrorRow messages={errorMessages} error={error} />
        <FormRow>
            <Field>
                <Submit label="Odeslat" form={form} loading={loading} />
            </Field>
        </FormRow>
    </Form>
}

function LengthError({form, name, length}) {
    if (!length) {
        length = 255
    }

    return <Error
        form={form}
        condition={data => data[name] && data[name].length > length}
    >
        <span>Obsah překračuje maximální povolenou délku {length} znaků.</span>
    </Error>
}
