import { useEffect, useState } from 'react'
import styled from 'styled-components'
import { BsSearch } from 'react-icons/bs'
import { AiFillPhone, AiOutlineMail } from 'react-icons/ai'
import { GoGlobe } from 'react-icons/go'
import { IoIosHand } from 'react-icons/io'
import { FaCity, FaMoneyBillWave } from 'react-icons/fa'
import { MdOutlineAlternateEmail } from 'react-icons/md'
import { ImManWoman } from 'react-icons/im'
import { device } from '../utils/media'
import { Highlight, useFilter } from './filter'
import List from './list'
import { get } from '../lib/backend.mjs'

export function Capacity() {
    const [error, setError] = useState(null)
    const [facilities, setFacilities] = useState(null)
    const filter = useFilter()

    useEffect(() => {
        get('/facility/list')
            .then(res => setFacilities(res.data))
            .catch(err => setError(err))
    }, [])

    const defaultType = { id: 0, name: 'Ostatní', position: 1000 }

    const types = (facilities || [])
        .filter(f => !f.deleted)
        .filter(f => f.facilityType && (f.facilityType.name == 'Noclehárny s nízkým prahem ∗' || f.facilityType.name == 'Zařízení 7/24 ∗'))
        .filter(f => filter.anyMatch([
            f.name,
            f.organization,
            f.address,
            f.facilityDistrict && f.facilityDistrict.name,
            f.facilityType && f.facilityType.name || defaultType.name]))
        .reduce((types, f) => {
            const typeId = f.facilityType && f.facilityType.id || 0
            types[typeId] = types[typeId] || f.facilityType || defaultType

            types[typeId].facilities = types[typeId].facilities || {}
            types[typeId].facilities[f.id] = f

            return types
        }, {})

    return <>
        <Filter filter={filter} />
        <Error error={error} />
        <Loading facilities={facilities}/>
        <FacilityTypeList types={types} filter={filter} />
    </>
}

function Filter({ filter }) {
    return <FilterWrapper>
        <BsSearch/>
        <Input onChange={e => filter.setQuery(e.target.value)} />
    </FilterWrapper>
}

const FilterWrapper = styled.label`
    display: flex;
    flex-wrap: nowrap;
    justify-content: flex-start;
    align-items: center;
    width: 100%;

    padding: 4px 1rem;
    border-radius: ${({ theme}) => theme.borderRadii.tile};
    margin: 1rem 0;
    transition: box-shadow ease 100ms, transform ease 150ms;

    & input {
        flex-grow: 1;
    }

    @media ${device.tablet} {
        width: 480px;
    }

    box-shadow: ${({ theme }) => theme.boxShadows.tile};
    &:hover {
        box-shadow: ${({ theme }) => theme.boxShadows.tileHover};
        transform: ${({ theme }) => theme.transforms.tileHover};
    }
`

const Input = styled.input`
    margin-left: 1rem;
    border-radius: ${({ theme}) => theme.borderRadii.tile};
    outline: none;
    border: none;
    padding: 4px 1rem;
`

function Error({ error }) {
    return error && <div>Chyba při načítání ze serveru ({error.message}).</div>
}

function Loading({ facilities }) {
    return !facilities && <div>Načítám...</div>
}

function FacilityTypeList({ types, filter }) {
    const facilityTypes = Object.values(types)

    return <List items={facilityTypes} item="type" order="position">
        <FacilityType filter={filter} />
    </List>
}

function FacilityType({ type, filter }) {
    const facilities = Object.values(type.facilities)

    return <FacilityTypeWrapper>
            <h2><Highlight filter={filter}>{type.name}</Highlight></h2>
            <List items={facilities} item="facility" order="sortOrder">
                <Facility filter={filter} />
            </List>
    </FacilityTypeWrapper>
}

const FacilityTypeWrapper = styled.div`
    margin-top: 1rem;

    & H2 {
        font-size: 1.1rem;
    }
`

function Facility({ facility, filter }) {
    const { name, organization, facilityDistrict, address, phone, email, url, targetGroup, restrictions, fee, note } = facility
    
    const counters = facility.counters
        .filter(c => !c.disabled)
        .filter(c => c.title)

    return <Item>
        <HeaderRow>
            <Title>
                <Highlight filter={filter}>{name}</Highlight>
            </Title>
            <Subtitle>
                <Highlight filter={filter}>{organization}</Highlight>
            </Subtitle>
        </HeaderRow>
        <InfoRow>
            <MaybeColumn icon={FaCity} condition={facilityDistrict}><Highlight filter={filter}>{facilityDistrict && facilityDistrict.name}</Highlight></MaybeColumn>
            <MaybeColumn icon={AiOutlineMail} condition={address}>
                <Highlight filter={filter}>{address}</Highlight>
            </MaybeColumn>
            <MaybeColumn icon={AiFillPhone}>{phone}</MaybeColumn>
            <MaybeColumn icon={MdOutlineAlternateEmail}>{email}</MaybeColumn>
            <MaybeColumn icon={GoGlobe} condition={url} scrollable>
                <a href={url} target="_blank">{url}</a>
            </MaybeColumn>
            <MaybeColumn icon={ImManWoman}>{targetGroup}</MaybeColumn>
            <MaybeColumn icon={IoIosHand}>{restrictions}</MaybeColumn>
            <MaybeColumn icon={FaMoneyBillWave}>{fee}</MaybeColumn>
            <MaybeColumn>{note}</MaybeColumn>
        </InfoRow>
        <CounterRow>
            <CounterWrapper className="label">obsazenost:</CounterWrapper>
            {
                counters.map((c, i) => <Counter counter={c} key={i} />)
            }
        </CounterRow>
    </Item>
}

function MaybeColumn(props) {
    const show = "condition" in props ? props.condition : props.children
    const Icon = props.icon
    
    return show &&
        <Column>
            <IconRow>
                <div className="icon">{props.icon && <Icon />}</div>
                <div>{props.children}</div>
            </IconRow>
        </Column>
}

const Item = styled.div`
    width: 100%;
    margin-top: 1rem;

    box-shadow: ${({ theme }) => theme.boxShadows.tile};
    border-radius: ${({ theme}) => theme.borderRadii.tile};
    padding: 1rem;
`

const Row = styled.div`
    display: flex;
    flex-wrap: wrap;
    justify-content: space-between;
`

const HeaderRow = styled(Row)`
    border-bottom: 1px solid ${({ theme }) => theme.colors.red};
    margin-bottom: 0.5rem;
`

const InfoRow = styled(Row)`
    font-size: 0.8em;

    & div {
        width: 100%;
    }
`

const CounterRow = styled(Row)`
    font-size: 0.9em;

    margin-top: 0.5rem;
    justify-content: end;
`

const Column = styled.div`
    margin: 1px 0 0 1px;
    background: ${({ theme }) => theme.colors.white};

    & .icon {
        display: inline-block;
        width: 1.5em;
        position: relative;
        top: 2px;
    }
`

const Title = styled(Column)`
    color: ${({ theme }) => theme.colors.red};
`

const Subtitle = styled(Column)`
`

const IconRow = styled.div`
    display: flex;
`

function Counter({ counter }) {
    return <CounterWrapper>
        <span>{counter.title}</span>
        <span>&nbsp;</span>
        <span>{counter.occupancy}</span>
        {
            counter.capacity > 0 && <span>/{counter.capacity}</span>
        }
    </CounterWrapper>
}

const CounterWrapper = styled.div`
    padding: 0px 4px;

    border: 1px solid ${({ theme }) => theme.colors.red};
    color: ${({ theme }) => theme.colors.white};
    background-color: ${({ theme }) => theme.colors.red};

    margin-top: 1px;

    & + & {
        margin-left: 0.5em;
    }

    &.label {
        border: 1px solid ${({ theme }) => theme.colors.red};
        color: ${({ theme }) => theme.colors.red};
        background-color: ${({ theme }) => theme.colors.white};
    }
`