import { useEffect, useState } from 'react'
import styled from 'styled-components'
import { BsChatRightDotsFill, BsChatRightDots } from 'react-icons/bs'
import { IoChatboxEllipsesOutline, IoChatboxEllipsesSharp } from 'react-icons/io5'
import theme from '../utils/theme'
import { device } from '../utils/media'
import { useAppContext } from '../service/ctx'
import { useInterval } from '../utils/hooks'

export default function Chat() {
    const [open, setOpen] = useState(null)
    const [hover, setHover] = useState(null)

    const { chatService } = useAppContext()

    return <>
        <Wrapper>
            <OpenButton
                    onClick={() => setOpen((o) => o ? null : 'open')}
                    onMouseEnter={() => setHover('hover')}
                    onMouseLeave={() => setHover(null)}
                    className={hover + ' ' + open}
                    tabIndex="0"
            >
                <Label>On-line poradna</Label>
                <Icon>
                    <IoChatboxEllipsesSharp />
                </Icon>
            </OpenButton>
        </Wrapper>
        <ChatWindow open={open} messages={chatService.getMessages()} onSubmit={chatService.submitMessage} />
    </>
}

const Wrapper = styled.div`
    position: relative;

    @media ${device.tablet} {
        min-width: 40%;
    }
`

const OpenButton = styled.button`
    position: absolute;
    top: -6px;
    right: 0;

    min-height: 2em;
    
    border: 2px solid ${({theme}) => theme.colors.grey};
    border-radius: ${({ theme}) => theme.borderRadii.tile};

    color: ${({theme}) => theme.colors.white};
    background-color: ${({theme}) => theme.colors.grey};
    
    display: flex;

    padding: 5px 3rem 5px 0;

    &.open {
        color: ${({theme}) => theme.colors.red};
        background-color: ${({theme}) => theme.colors.white};
    }

    &.hover {
        border-color: ${({theme}) => theme.colors.white};
    }

    &.open.hover {
        border-color: ${({theme}) => theme.colors.red};
    }
`

const Label = styled.div`
    display: none;

    padding-left: 0.5rem;

    @media ${device.tablet} {
        display: block;
    }
`

const Icon = styled.div`
    display: inline-block;
    position: absolute;
    top: -2px;
    right: 0;
    font-size: 2rem;
    padding: 0 0.5rem 0 0;
`

function ChatWindow({ open, messages, onSubmit }) {
    const [message, setMessage] = useState("")
    const [loading, setLoading] = useState(false)
    const [error, setError] = useState(null)

    const { chatService } = useAppContext()

    useInterval(() => chatService.fetchMessages())

    const handleSuccess = () => {
        setMessage("")
        setError(null)
        setLoading(false)
    }

    const handleError = (e) => {
        setError("Odeslání zprávy se nezdařilo.")
        setLoading(false)
    }

    const handleClick = () => {
        setLoading(true)
        onSubmit(message, handleSuccess, handleError)
    }

    useEffect(() => {
        const chatMessages = document.getElementsByClassName("chatMessages")[0]
        if (chatMessages) {
            chatMessages.scrollTop = chatMessages.scrollHeight
        }

        const textArea = document.getElementsByClassName("chatTextarea")[0]
        if (textArea) {
            textArea.focus()
        }
    })

    return open && <WindowWrapper>
        <Window>
            <Info>
                Chat je v provozu ve všední dny 8 - 10, 16 - 18 hod.
            </Info>
            <Messages className="chatMessages">
                {
                    messages.map((m, i) => <Message message={m} key={i} />)
                }
            </Messages>
            <MessageForm>
                <Textarea
                    value={message}
                    onChange={e => setMessage(e.target.value)}
                    disabled={loading}
                    className="chatTextarea" />
                <SendButton
                    onClick={handleClick}
                    disabled={!message}
                    tabIndex="0">Odeslat</SendButton>
            </MessageForm>
        </Window>
    </WindowWrapper>
}

const WindowWrapper = styled.div`
    position: absolute;
    top: 3.5em;
    right: 0;

    z-index: 100;

    width: 100%;
    max-width: 700px;

    padding: 0.5em;
`

const Window = styled.div`
    height: 50vh;
    min-height: 300px;

    background-color: ${({ theme}) => theme.colors.grey};
    
    border-radius: ${({ theme}) => theme.borderRadii.tile};
    box-shadow: ${({ theme }) => theme.boxShadows.tile};

    display: flex;
    flex-direction: column;

    text-align: left;
    font-size: 0.8em;

    & div {
        padding: 0.5rem;
    }
`

const Info = styled.div`
`

const Messages = styled.div`
    flex-grow: 1;
    overflow: scroll;
`

function Message({ message }) {
    const className = message.username ? 'answer' : 'question'
    return <MessageDiv className={className}>
        <span>{message.text}</span>
        <span>{new Date(message.posted).toLocaleString()}</span>
    </MessageDiv>
}

const MessageDiv = styled.div`
    border-radius: ${({ theme}) => theme.borderRadii.tile};
    background-color: ${({theme}) => theme.colors.white};
    color: ${({theme}) => theme.colors.grey};
    
    display: flex;
    justify-content: space-between;        

    & + & {
        margin-top: 0.5rem;
    }

    &.question {
        margin-left: 2rem;
    }

    &.answer {
        margin-right: 2rem;
    }

    position: relative;
    
    &.question:after {
        content: '';
        position: absolute;
        right: 12px;
        bottom: 4px;
        width: 0;
        height: 0;
        border: 12px solid transparent;
        border-left-color: white;
        border-right: 0;
        border-bottom: 0;
        margin-top: -10px;
        margin-right: -20px;
    }

    &.answer:before {
        content: '';
        position: absolute;
        left: 12px;
        top: 14px;
        width: 0;
        height: 0;
        border: 12px solid transparent;
        border-right-color: white;
        border-left: 0;
        border-top: 0;
        margin-top: -10.5px;
        margin-left: -21px;
    }
`

const MessageForm = styled.div`
`

const Textarea = styled.textarea`
    border-radius: ${({ theme}) => theme.borderRadii.tile};
    width: 100%;
    resize: none;
    padding: 0.5em;
    min-height: 4em;

    border: 2px solid ${({theme}) => theme.colors.white};
    &:focus {
        border-color: ${({theme}) => theme.colors.red};
        outline: none;
    }
    &:disabled {
        background: ${({theme}) => theme.colors.white};
        border-color: ${({theme}) => theme.colors.white};
        color: ${({theme}) => theme.colors.grey}
    }
`

const SendButton = styled.button`
    border: 2px solid ${({theme}) => theme.colors.white};
    border-radius: ${({ theme}) => theme.borderRadii.tile};

    float: right;
    
    color: ${({theme}) => theme.colors.red};
    font-weight: bold;
    
    background-color: ${({theme}) => theme.colors.white};
    padding: 3px 1rem 1px 1rem;

    &:hover {
        border-color: ${({theme}) => theme.colors.red};
    }
    &:focus {
        border-color: ${({theme}) => theme.colors.red};
        outline: none;
    }
    &:disabled {
        color: ${({theme}) => theme.colors.grey};
    }
`