import axios from "axios"
import he from "he"

const PAGES_URL = 'https://cms.bezdomova.eu/?rest_route=/wp/v2/pages&per_page=100'
const NEWS_URL = 'https://cms.bezdomova.eu/?rest_route=/wp/v2/posts&per_page=100'

const BASE_URI = 'https://cms.bezdomova.eu/index.php/v-praze-bez-domova'

let fetched = false
const pages = {}
const contents = {}

export async function lastModified() {
    const cmsPages = await fetchPagesFromCms()
    const cmsNews = await fetchNewsFromCms()
    const allPages = cmsPages.concat(cmsNews)

    return allPages
        .map(p => new Date(p.modified))
        .reduce((a, b) => a > b ? a : b)
}

async function fetchPagesFromCms() {
    return await axios
        .get(PAGES_URL)
        .then(res => res.data)
}

async function fetchNewsFromCms() {
    return await axios
        .get(NEWS_URL)
        .then(res => res.data)
}

export async function getPaths() {
    await assureFetched()
    
    return Object.values(pages)
        .filter(p => p.length)
        .map(p => p.path)
}

async function assureFetched() {
    if (fetched) {
        return
    }

    const cmsPages = await fetchPagesFromCms()

    for (let p of cmsPages) {
        let path = p.parent == 0
            ? '/'
            : sanitize(p.link.substring(BASE_URI.length))

        pages[path] = {
            path: path,
            title: he.decode(p.title.rendered),
            order: p.menu_order,
            template: p.template,
            icon: p.icon,

            href: p.href,
            background: p.background,

            length: p.content.rendered.length
        }

        contents[path] = {
            content: p.content.rendered
        }
    }

    const cmsNews = await fetchNewsFromCms()

    for (const n of cmsNews) {
        const path = sanitize('/novinky/' + n.slug)

        pages[path] = {
            path: path,
            title: he.decode(n.title.rendered),
            excerpt: n.excerpt.rendered,
    
            order: 0,
            template: n.template,
            icon: null,
    
            href: null,
            background: null,
    
            length: n.content.rendered.length
        }

        contents[path] = {
            content: n.content.rendered
        }
    }

    fetched = true
}

function sanitize(str) {
    str = str.split('#')[0]

    if (str.endsWith('/')) {
        str = str.substring(0, str.length - 1)
    }

    if (!str.startsWith('/')) {
        str = '/' + str
    }

    return str
}

export async function getByPath(path) {
    await assureFetched()
    
    return {
        ...pages[path],
        ...contents[path]
    }
}

export async function getPages() {
    await assureFetched()
    
    return pages
}

export async function getNews() {
    await assureFetched()

    return Object
        .values(pages)
        .filter(p => p.path.startsWith('/novinky'))
}
